from decimal import Decimal
import six

PICONERO = Decimal("0.000000000001")
EMPTY_KEY = "0" * 64


def to_atomic(amount):
    """Convert Monero decimal to atomic integer of piconero."""
    if not isinstance(amount, (Decimal, float) + six.integer_types):
        raise ValueError(
            "Amount '{}' doesn't have numeric type. Only Decimal, int, long and "
            "float (not recommended) are accepted as amounts."
        )
    return int(amount * 10 ** 12)


def from_atomic(amount):
    """Convert atomic integer of piconero to Monero decimal."""
    return (Decimal(amount) * PICONERO).quantize(PICONERO)


def as_monero(amount):
    """Return the amount rounded to maximal Monero precision."""
    return Decimal(amount).quantize(PICONERO)


class PaymentID(object):
    """
    A class that validates Monero payment ID.

    Payment IDs can be used as str or int across the module, however this class
    offers validation as well as simple conversion and comparison to those two
    primitive types.

    :param payment_id: the payment ID as integer or hexadecimal string
    """

    _payment_id = None

    def __init__(self, payment_id):
        if isinstance(payment_id, PaymentID):
            payment_id = int(payment_id)
        if isinstance(payment_id, six.text_type) or isinstance(
            payment_id, six.string_types
        ):
            payment_id = int(payment_id, 16)
        elif not isinstance(payment_id, six.integer_types):
            raise TypeError(
                "payment_id must be either int or hexadecimal str or bytes, "
                "is {0}".format(type(payment_id))
            )
        if payment_id.bit_length() > 256:
            raise ValueError(
                "payment_id {0} is more than 256 bits long".format(payment_id)
            )
        self._payment_id = payment_id

    def is_short(self):
        """Returns True if payment ID is short enough to be included
        in :class:`IntegratedAddress <monero.address.IntegratedAddress>`."""
        return self._payment_id.bit_length() <= 64

    def __repr__(self):
        if self.is_short():
            return "{:016x}".format(self._payment_id)
        return "{:064x}".format(self._payment_id)

    def __int__(self):
        return self._payment_id

    def __eq__(self, other):
        if isinstance(other, PaymentID):
            return int(self) == int(other)
        elif isinstance(other, six.integer_types):
            return int(self) == other
        elif isinstance(other, six.text_type) or isinstance(other, six.string_types):
            return str(self) == six.ensure_str(other)
        return super(PaymentID, self).__eq__(other)



from ... import exceptions


class RPCError(exceptions.BackendException):
    pass


class Unauthorized(RPCError):
    pass


class MethodNotFound(RPCError):
    pass


class RestrictedRPC(RPCError):
    pass



#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# Word list originally created by dabura667 and released under The MIT License (MIT)
#
# The MIT License (MIT)
#
# Permission is hereby granted, free of charge, to any person obtaining
# a copy of this software and associated documentation files (the
# "Software"), to deal in the Software without restriction, including
# without limitation the rights to use, copy, modify, merge, publish,
# distribute, sublicense, and/or sell copies of the Software, and to
# permit persons to whom the Software is furnished to do so, subject to
# the following conditions:
#
# The above copyright notice and this permission notice shall be
# included in all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
# EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
# MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
# NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE
# LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION
# OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION
# WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
#
# Code surrounding the word list is Copyright (c) 2014-2018, The Monero Project
#
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without modification, are
# permitted provided that the following conditions are met:
#
# 1. Redistributions of source code must retain the above copyright notice, this list of
#    conditions and the following disclaimer.
#
# 2. Redistributions in binary form must reproduce the above copyright notice, this list
#    of conditions and the following disclaimer in the documentation and/or other
#    materials provided with the distribution.
#
# 3. Neither the name of the copyright holder nor the names of its contributors may be
#    used to endorse or promote products derived from this software without specific
#    prior written permission.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY
# EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
# MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL
# THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
# SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
# PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
# INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
# STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF
# THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
#
# list of words from:
#     https://github.com/monero-project/monero/blob/master/src/mnemonics/chinese_simplified.h
#     Most recent commit: 8ea3c4d54429976e9001b8cc4868b721e6747e3c


from .wordlist import Wordlist


class ChineseSimplified(Wordlist):
    language_name = (u"简体中文 (中国)",)
    english_language_name = "Chinese (simplified)"
    unique_prefix_length = 1
    word_list = [
        u"的",
        u"一",
        u"是",
        u"在",
        u"不",
        u"了",
        u"有",
        u"和",
        u"人",
        u"这",
        u"中",
        u"大",
        u"为",
        u"上",
        u"个",
        u"国",
        u"我",
        u"以",
        u"要",
        u"他",
        u"时",
        u"来",
        u"用",
        u"们",
        u"生",
        u"到",
        u"作",
        u"地",
        u"于",
        u"出",
        u"就",
        u"分",
        u"对",
        u"成",
        u"会",
        u"可",
        u"主",
        u"发",
        u"年",
        u"动",
        u"同",
        u"工",
        u"也",
        u"能",
        u"下",
        u"过",
        u"子",
        u"说",
        u"产",
        u"种",
        u"面",
        u"而",
        u"方",
        u"后",
        u"多",
        u"定",
        u"行",
        u"学",
        u"法",
        u"所",
        u"民",
        u"得",
        u"经",
        u"十",
        u"三",
        u"之",
        u"进",
        u"着",
        u"等",
        u"部",
        u"度",
        u"家",
        u"电",
        u"力",
        u"里",
        u"如",
        u"水",
        u"化",
        u"高",
        u"自",
        u"二",
        u"理",
        u"起",
        u"小",
        u"物",
        u"现",
        u"实",
        u"加",
        u"量",
        u"都",
        u"两",
        u"体",
        u"制",
        u"机",
        u"当",
        u"使",
        u"点",
        u"从",
        u"业",
        u"本",
        u"去",
        u"把",
        u"性",
        u"好",
        u"应",
        u"开",
        u"它",
        u"合",
        u"还",
        u"因",
        u"由",
        u"其",
        u"些",
        u"然",
        u"前",
        u"外",
        u"天",
        u"政",
        u"四",
        u"日",
        u"那",
        u"社",
        u"义",
        u"事",
        u"平",
        u"形",
        u"相",
        u"全",
        u"表",
        u"间",
        u"样",
        u"与",
        u"关",
        u"各",
        u"重",
        u"新",
        u"线",
        u"内",
        u"数",
        u"正",
        u"心",
        u"反",
        u"你",
        u"明",
        u"看",
        u"原",
        u"又",
        u"么",
        u"利",
        u"比",
        u"或",
        u"但",
        u"质",
        u"气",
        u"第",
        u"向",
        u"道",
        u"命",
        u"此",
        u"变",
        u"条",
        u"只",
        u"没",
        u"结",
        u"解",
        u"问",
        u"意",
        u"建",
        u"月",
        u"公",
        u"无",
        u"系",
        u"军",
        u"很",
        u"情",
        u"者",
        u"最",
        u"立",
        u"代",
        u"想",
        u"已",
        u"通",
        u"并",
        u"提",
        u"直",
        u"题",
        u"党",
        u"程",
        u"展",
        u"五",
        u"果",
        u"料",
        u"象",
        u"员",
        u"革",
        u"位",
        u"入",
        u"常",
        u"文",
        u"总",
        u"次",
        u"品",
        u"式",
        u"活",
        u"设",
        u"及",
        u"管",
        u"特",
        u"件",
        u"长",
        u"求",
        u"老",
        u"头",
        u"基",
        u"资",
        u"边",
        u"流",
        u"路",
        u"级",
        u"少",
        u"图",
        u"山",
        u"统",
        u"接",
        u"知",
        u"较",
        u"将",
        u"组",
        u"见",
        u"计",
        u"别",
        u"她",
        u"手",
        u"角",
        u"期",
        u"根",
        u"论",
        u"运",
        u"农",
        u"指",
        u"几",
        u"九",
        u"区",
        u"强",
        u"放",
        u"决",
        u"西",
        u"被",
        u"干",
        u"做",
        u"必",
        u"战",
        u"先",
        u"回",
        u"则",
        u"任",
        u"取",
        u"据",
        u"处",
        u"队",
        u"南",
        u"给",
        u"色",
        u"光",
        u"门",
        u"即",
        u"保",
        u"治",
        u"北",
        u"造",
        u"百",
        u"规",
        u"热",
        u"领",
        u"七",
        u"海",
        u"口",
        u"东",
        u"导",
        u"器",
        u"压",
        u"志",
        u"世",
        u"金",
        u"增",
        u"争",
        u"济",
        u"阶",
        u"油",
        u"思",
        u"术",
        u"极",
        u"交",
        u"受",
        u"联",
        u"什",
        u"认",
        u"六",
        u"共",
        u"权",
        u"收",
        u"证",
        u"改",
        u"清",
        u"美",
        u"再",
        u"采",
        u"转",
        u"更",
        u"单",
        u"风",
        u"切",
        u"打",
        u"白",
        u"教",
        u"速",
        u"花",
        u"带",
        u"安",
        u"场",
        u"身",
        u"车",
        u"例",
        u"真",
        u"务",
        u"具",
        u"万",
        u"每",
        u"目",
        u"至",
        u"达",
        u"走",
        u"积",
        u"示",
        u"议",
        u"声",
        u"报",
        u"斗",
        u"完",
        u"类",
        u"八",
        u"离",
        u"华",
        u"名",
        u"确",
        u"才",
        u"科",
        u"张",
        u"信",
        u"马",
        u"节",
        u"话",
        u"米",
        u"整",
        u"空",
        u"元",
        u"况",
        u"今",
        u"集",
        u"温",
        u"传",
        u"土",
        u"许",
        u"步",
        u"群",
        u"广",
        u"石",
        u"记",
        u"需",
        u"段",
        u"研",
        u"界",
        u"拉",
        u"林",
        u"律",
        u"叫",
        u"且",
        u"究",
        u"观",
        u"越",
        u"织",
        u"装",
        u"影",
        u"算",
        u"低",
        u"持",
        u"音",
        u"众",
        u"书",
        u"布",
        u"复",
        u"容",
        u"儿",
        u"须",
        u"际",
        u"商",
        u"非",
        u"验",
        u"连",
        u"断",
        u"深",
        u"难",
        u"近",
        u"矿",
        u"千",
        u"周",
        u"委",
        u"素",
        u"技",
        u"备",
        u"半",
        u"办",
        u"青",
        u"省",
        u"列",
        u"习",
        u"响",
        u"约",
        u"支",
        u"般",
        u"史",
        u"感",
        u"劳",
        u"便",
        u"团",
        u"往",
        u"酸",
        u"历",
        u"市",
        u"克",
        u"何",
        u"除",
        u"消",
        u"构",
        u"府",
        u"称",
        u"太",
        u"准",
        u"精",
        u"值",
        u"号",
        u"率",
        u"族",
        u"维",
        u"划",
        u"选",
        u"标",
        u"写",
        u"存",
        u"候",
        u"毛",
        u"亲",
        u"快",
        u"效",
        u"斯",
        u"院",
        u"查",
        u"江",
        u"型",
        u"眼",
        u"王",
        u"按",
        u"格",
        u"养",
        u"易",
        u"置",
        u"派",
        u"层",
        u"片",
        u"始",
        u"却",
        u"专",
        u"状",
        u"育",
        u"厂",
        u"京",
        u"识",
        u"适",
        u"属",
        u"圆",
        u"包",
        u"火",
        u"住",
        u"调",
        u"满",
        u"县",
        u"局",
        u"照",
        u"参",
        u"红",
        u"细",
        u"引",
        u"听",
        u"该",
        u"铁",
        u"价",
        u"严",
        u"首",
        u"底",
        u"液",
        u"官",
        u"德",
        u"随",
        u"病",
        u"苏",
        u"失",
        u"尔",
        u"死",
        u"讲",
        u"配",
        u"女",
        u"黄",
        u"推",
        u"显",
        u"谈",
        u"罪",
        u"神",
        u"艺",
        u"呢",
        u"席",
        u"含",
        u"企",
        u"望",
        u"密",
        u"批",
        u"营",
        u"项",
        u"防",
        u"举",
        u"球",
        u"英",
        u"氧",
        u"势",
        u"告",
        u"李",
        u"台",
        u"落",
        u"木",
        u"帮",
        u"轮",
        u"破",
        u"亚",
        u"师",
        u"围",
        u"注",
        u"远",
        u"字",
        u"材",
        u"排",
        u"供",
        u"河",
        u"态",
        u"封",
        u"另",
        u"施",
        u"减",
        u"树",
        u"溶",
        u"怎",
        u"止",
        u"案",
        u"言",
        u"士",
        u"均",
        u"武",
        u"固",
        u"叶",
        u"鱼",
        u"波",
        u"视",
        u"仅",
        u"费",
        u"紧",
        u"爱",
        u"左",
        u"章",
        u"早",
        u"朝",
        u"害",
        u"续",
        u"轻",
        u"服",
        u"试",
        u"食",
        u"充",
        u"兵",
        u"源",
        u"判",
        u"护",
        u"司",
        u"足",
        u"某",
        u"练",
        u"差",
        u"致",
        u"板",
        u"田",
        u"降",
        u"黑",
        u"犯",
        u"负",
        u"击",
        u"范",
        u"继",
        u"兴",
        u"似",
        u"余",
        u"坚",
        u"曲",
        u"输",
        u"修",
        u"故",
        u"城",
        u"夫",
        u"够",
        u"送",
        u"笔",
        u"船",
        u"占",
        u"右",
        u"财",
        u"吃",
        u"富",
        u"春",
        u"职",
        u"觉",
        u"汉",
        u"画",
        u"功",
        u"巴",
        u"跟",
        u"虽",
        u"杂",
        u"飞",
        u"检",
        u"吸",
        u"助",
        u"升",
        u"阳",
        u"互",
        u"初",
        u"创",
        u"抗",
        u"考",
        u"投",
        u"坏",
        u"策",
        u"古",
        u"径",
        u"换",
        u"未",
        u"跑",
        u"留",
        u"钢",
        u"曾",
        u"端",
        u"责",
        u"站",
        u"简",
        u"述",
        u"钱",
        u"副",
        u"尽",
        u"帝",
        u"射",
        u"草",
        u"冲",
        u"承",
        u"独",
        u"令",
        u"限",
        u"阿",
        u"宣",
        u"环",
        u"双",
        u"请",
        u"超",
        u"微",
        u"让",
        u"控",
        u"州",
        u"良",
        u"轴",
        u"找",
        u"否",
        u"纪",
        u"益",
        u"依",
        u"优",
        u"顶",
        u"础",
        u"载",
        u"倒",
        u"房",
        u"突",
        u"坐",
        u"粉",
        u"敌",
        u"略",
        u"客",
        u"袁",
        u"冷",
        u"胜",
        u"绝",
        u"析",
        u"块",
        u"剂",
        u"测",
        u"丝",
        u"协",
        u"诉",
        u"念",
        u"陈",
        u"仍",
        u"罗",
        u"盐",
        u"友",
        u"洋",
        u"错",
        u"苦",
        u"夜",
        u"刑",
        u"移",
        u"频",
        u"逐",
        u"靠",
        u"混",
        u"母",
        u"短",
        u"皮",
        u"终",
        u"聚",
        u"汽",
        u"村",
        u"云",
        u"哪",
        u"既",
        u"距",
        u"卫",
        u"停",
        u"烈",
        u"央",
        u"察",
        u"烧",
        u"迅",
        u"境",
        u"若",
        u"印",
        u"洲",
        u"刻",
        u"括",
        u"激",
        u"孔",
        u"搞",
        u"甚",
        u"室",
        u"待",
        u"核",
        u"校",
        u"散",
        u"侵",
        u"吧",
        u"甲",
        u"游",
        u"久",
        u"菜",
        u"味",
        u"旧",
        u"模",
        u"湖",
        u"货",
        u"损",
        u"预",
        u"阻",
        u"毫",
        u"普",
        u"稳",
        u"乙",
        u"妈",
        u"植",
        u"息",
        u"扩",
        u"银",
        u"语",
        u"挥",
        u"酒",
        u"守",
        u"拿",
        u"序",
        u"纸",
        u"医",
        u"缺",
        u"雨",
        u"吗",
        u"针",
        u"刘",
        u"啊",
        u"急",
        u"唱",
        u"误",
        u"训",
        u"愿",
        u"审",
        u"附",
        u"获",
        u"茶",
        u"鲜",
        u"粮",
        u"斤",
        u"孩",
        u"脱",
        u"硫",
        u"肥",
        u"善",
        u"龙",
        u"演",
        u"父",
        u"渐",
        u"血",
        u"欢",
        u"械",
        u"掌",
        u"歌",
        u"沙",
        u"刚",
        u"攻",
        u"谓",
        u"盾",
        u"讨",
        u"晚",
        u"粒",
        u"乱",
        u"燃",
        u"矛",
        u"乎",
        u"杀",
        u"药",
        u"宁",
        u"鲁",
        u"贵",
        u"钟",
        u"煤",
        u"读",
        u"班",
        u"伯",
        u"香",
        u"介",
        u"迫",
        u"句",
        u"丰",
        u"培",
        u"握",
        u"兰",
        u"担",
        u"弦",
        u"蛋",
        u"沉",
        u"假",
        u"穿",
        u"执",
        u"答",
        u"乐",
        u"谁",
        u"顺",
        u"烟",
        u"缩",
        u"征",
        u"脸",
        u"喜",
        u"松",
        u"脚",
        u"困",
        u"异",
        u"免",
        u"背",
        u"星",
        u"福",
        u"买",
        u"染",
        u"井",
        u"概",
        u"慢",
        u"怕",
        u"磁",
        u"倍",
        u"祖",
        u"皇",
        u"促",
        u"静",
        u"补",
        u"评",
        u"翻",
        u"肉",
        u"践",
        u"尼",
        u"衣",
        u"宽",
        u"扬",
        u"棉",
        u"希",
        u"伤",
        u"操",
        u"垂",
        u"秋",
        u"宜",
        u"氢",
        u"套",
        u"督",
        u"振",
        u"架",
        u"亮",
        u"末",
        u"宪",
        u"庆",
        u"编",
        u"牛",
        u"触",
        u"映",
        u"雷",
        u"销",
        u"诗",
        u"座",
        u"居",
        u"抓",
        u"裂",
        u"胞",
        u"呼",
        u"娘",
        u"景",
        u"威",
        u"绿",
        u"晶",
        u"厚",
        u"盟",
        u"衡",
        u"鸡",
        u"孙",
        u"延",
        u"危",
        u"胶",
        u"屋",
        u"乡",
        u"临",
        u"陆",
        u"顾",
        u"掉",
        u"呀",
        u"灯",
        u"岁",
        u"措",
        u"束",
        u"耐",
        u"剧",
        u"玉",
        u"赵",
        u"跳",
        u"哥",
        u"季",
        u"课",
        u"凯",
        u"胡",
        u"额",
        u"款",
        u"绍",
        u"卷",
        u"齐",
        u"伟",
        u"蒸",
        u"殖",
        u"永",
        u"宗",
        u"苗",
        u"川",
        u"炉",
        u"岩",
        u"弱",
        u"零",
        u"杨",
        u"奏",
        u"沿",
        u"露",
        u"杆",
        u"探",
        u"滑",
        u"镇",
        u"饭",
        u"浓",
        u"航",
        u"怀",
        u"赶",
        u"库",
        u"夺",
        u"伊",
        u"灵",
        u"税",
        u"途",
        u"灭",
        u"赛",
        u"归",
        u"召",
        u"鼓",
        u"播",
        u"盘",
        u"裁",
        u"险",
        u"康",
        u"唯",
        u"录",
        u"菌",
        u"纯",
        u"借",
        u"糖",
        u"盖",
        u"横",
        u"符",
        u"私",
        u"努",
        u"堂",
        u"域",
        u"枪",
        u"润",
        u"幅",
        u"哈",
        u"竟",
        u"熟",
        u"虫",
        u"泽",
        u"脑",
        u"壤",
        u"碳",
        u"欧",
        u"遍",
        u"侧",
        u"寨",
        u"敢",
        u"彻",
        u"虑",
        u"斜",
        u"薄",
        u"庭",
        u"纳",
        u"弹",
        u"饲",
        u"伸",
        u"折",
        u"麦",
        u"湿",
        u"暗",
        u"荷",
        u"瓦",
        u"塞",
        u"床",
        u"筑",
        u"恶",
        u"户",
        u"访",
        u"塔",
        u"奇",
        u"透",
        u"梁",
        u"刀",
        u"旋",
        u"迹",
        u"卡",
        u"氯",
        u"遇",
        u"份",
        u"毒",
        u"泥",
        u"退",
        u"洗",
        u"摆",
        u"灰",
        u"彩",
        u"卖",
        u"耗",
        u"夏",
        u"择",
        u"忙",
        u"铜",
        u"献",
        u"硬",
        u"予",
        u"繁",
        u"圈",
        u"雪",
        u"函",
        u"亦",
        u"抽",
        u"篇",
        u"阵",
        u"阴",
        u"丁",
        u"尺",
        u"追",
        u"堆",
        u"雄",
        u"迎",
        u"泛",
        u"爸",
        u"楼",
        u"避",
        u"谋",
        u"吨",
        u"野",
        u"猪",
        u"旗",
        u"累",
        u"偏",
        u"典",
        u"馆",
        u"索",
        u"秦",
        u"脂",
        u"潮",
        u"爷",
        u"豆",
        u"忽",
        u"托",
        u"惊",
        u"塑",
        u"遗",
        u"愈",
        u"朱",
        u"替",
        u"纤",
        u"粗",
        u"倾",
        u"尚",
        u"痛",
        u"楚",
        u"谢",
        u"奋",
        u"购",
        u"磨",
        u"君",
        u"池",
        u"旁",
        u"碎",
        u"骨",
        u"监",
        u"捕",
        u"弟",
        u"暴",
        u"割",
        u"贯",
        u"殊",
        u"释",
        u"词",
        u"亡",
        u"壁",
        u"顿",
        u"宝",
        u"午",
        u"尘",
        u"闻",
        u"揭",
        u"炮",
        u"残",
        u"冬",
        u"桥",
        u"妇",
        u"警",
        u"综",
        u"招",
        u"吴",
        u"付",
        u"浮",
        u"遭",
        u"徐",
        u"您",
        u"摇",
        u"谷",
        u"赞",
        u"箱",
        u"隔",
        u"订",
        u"男",
        u"吹",
        u"园",
        u"纷",
        u"唐",
        u"败",
        u"宋",
        u"玻",
        u"巨",
        u"耕",
        u"坦",
        u"荣",
        u"闭",
        u"湾",
        u"键",
        u"凡",
        u"驻",
        u"锅",
        u"救",
        u"恩",
        u"剥",
        u"凝",
        u"碱",
        u"齿",
        u"截",
        u"炼",
        u"麻",
        u"纺",
        u"禁",
        u"废",
        u"盛",
        u"版",
        u"缓",
        u"净",
        u"睛",
        u"昌",
        u"婚",
        u"涉",
        u"筒",
        u"嘴",
        u"插",
        u"岸",
        u"朗",
        u"庄",
        u"街",
        u"藏",
        u"姑",
        u"贸",
        u"腐",
        u"奴",
        u"啦",
        u"惯",
        u"乘",
        u"伙",
        u"恢",
        u"匀",
        u"纱",
        u"扎",
        u"辩",
        u"耳",
        u"彪",
        u"臣",
        u"亿",
        u"璃",
        u"抵",
        u"脉",
        u"秀",
        u"萨",
        u"俄",
        u"网",
        u"舞",
        u"店",
        u"喷",
        u"纵",
        u"寸",
        u"汗",
        u"挂",
        u"洪",
        u"贺",
        u"闪",
        u"柬",
        u"爆",
        u"烯",
        u"津",
        u"稻",
        u"墙",
        u"软",
        u"勇",
        u"像",
        u"滚",
        u"厘",
        u"蒙",
        u"芳",
        u"肯",
        u"坡",
        u"柱",
        u"荡",
        u"腿",
        u"仪",
        u"旅",
        u"尾",
        u"轧",
        u"冰",
        u"贡",
        u"登",
        u"黎",
        u"削",
        u"钻",
        u"勒",
        u"逃",
        u"障",
        u"氨",
        u"郭",
        u"峰",
        u"币",
        u"港",
        u"伏",
        u"轨",
        u"亩",
        u"毕",
        u"擦",
        u"莫",
        u"刺",
        u"浪",
        u"秘",
        u"援",
        u"株",
        u"健",
        u"售",
        u"股",
        u"岛",
        u"甘",
        u"泡",
        u"睡",
        u"童",
        u"铸",
        u"汤",
        u"阀",
        u"休",
        u"汇",
        u"舍",
        u"牧",
        u"绕",
        u"炸",
        u"哲",
        u"磷",
        u"绩",
        u"朋",
        u"淡",
        u"尖",
        u"启",
        u"陷",
        u"柴",
        u"呈",
        u"徒",
        u"颜",
        u"泪",
        u"稍",
        u"忘",
        u"泵",
        u"蓝",
        u"拖",
        u"洞",
        u"授",
        u"镜",
        u"辛",
        u"壮",
        u"锋",
        u"贫",
        u"虚",
        u"弯",
        u"摩",
        u"泰",
        u"幼",
        u"廷",
        u"尊",
        u"窗",
        u"纲",
        u"弄",
        u"隶",
        u"疑",
        u"氏",
        u"宫",
        u"姐",
        u"震",
        u"瑞",
        u"怪",
        u"尤",
        u"琴",
        u"循",
        u"描",
        u"膜",
        u"违",
        u"夹",
        u"腰",
        u"缘",
        u"珠",
        u"穷",
        u"森",
        u"枝",
        u"竹",
        u"沟",
        u"催",
        u"绳",
        u"忆",
        u"邦",
        u"剩",
        u"幸",
        u"浆",
        u"栏",
        u"拥",
        u"牙",
        u"贮",
        u"礼",
        u"滤",
        u"钠",
        u"纹",
        u"罢",
        u"拍",
        u"咱",
        u"喊",
        u"袖",
        u"埃",
        u"勤",
        u"罚",
        u"焦",
        u"潜",
        u"伍",
        u"墨",
        u"欲",
        u"缝",
        u"姓",
        u"刊",
        u"饱",
        u"仿",
        u"奖",
        u"铝",
        u"鬼",
        u"丽",
        u"跨",
        u"默",
        u"挖",
        u"链",
        u"扫",
        u"喝",
        u"袋",
        u"炭",
        u"污",
        u"幕",
        u"诸",
        u"弧",
        u"励",
        u"梅",
        u"奶",
        u"洁",
        u"灾",
        u"舟",
        u"鉴",
        u"苯",
        u"讼",
        u"抱",
        u"毁",
        u"懂",
        u"寒",
        u"智",
        u"埔",
        u"寄",
        u"届",
        u"跃",
        u"渡",
        u"挑",
        u"丹",
        u"艰",
        u"贝",
        u"碰",
        u"拔",
        u"爹",
        u"戴",
        u"码",
        u"梦",
        u"芽",
        u"熔",
        u"赤",
        u"渔",
        u"哭",
        u"敬",
        u"颗",
        u"奔",
        u"铅",
        u"仲",
        u"虎",
        u"稀",
        u"妹",
        u"乏",
        u"珍",
        u"申",
        u"桌",
        u"遵",
        u"允",
        u"隆",
        u"螺",
        u"仓",
        u"魏",
        u"锐",
        u"晓",
        u"氮",
        u"兼",
        u"隐",
        u"碍",
        u"赫",
        u"拨",
        u"忠",
        u"肃",
        u"缸",
        u"牵",
        u"抢",
        u"博",
        u"巧",
        u"壳",
        u"兄",
        u"杜",
        u"讯",
        u"诚",
        u"碧",
        u"祥",
        u"柯",
        u"页",
        u"巡",
        u"矩",
        u"悲",
        u"灌",
        u"龄",
        u"伦",
        u"票",
        u"寻",
        u"桂",
        u"铺",
        u"圣",
        u"恐",
        u"恰",
        u"郑",
        u"趣",
        u"抬",
        u"荒",
        u"腾",
        u"贴",
        u"柔",
        u"滴",
        u"猛",
        u"阔",
        u"辆",
        u"妻",
        u"填",
        u"撤",
        u"储",
        u"签",
        u"闹",
        u"扰",
        u"紫",
        u"砂",
        u"递",
        u"戏",
        u"吊",
        u"陶",
        u"伐",
        u"喂",
        u"疗",
        u"瓶",
        u"婆",
        u"抚",
        u"臂",
        u"摸",
        u"忍",
        u"虾",
        u"蜡",
        u"邻",
        u"胸",
        u"巩",
        u"挤",
        u"偶",
        u"弃",
        u"槽",
        u"劲",
        u"乳",
        u"邓",
        u"吉",
        u"仁",
        u"烂",
        u"砖",
        u"租",
        u"乌",
        u"舰",
        u"伴",
        u"瓜",
        u"浅",
        u"丙",
        u"暂",
        u"燥",
        u"橡",
        u"柳",
        u"迷",
        u"暖",
        u"牌",
        u"秧",
        u"胆",
        u"详",
        u"簧",
        u"踏",
        u"瓷",
        u"谱",
        u"呆",
        u"宾",
        u"糊",
        u"洛",
        u"辉",
        u"愤",
        u"竞",
        u"隙",
        u"怒",
        u"粘",
        u"乃",
        u"绪",
        u"肩",
        u"籍",
        u"敏",
        u"涂",
        u"熙",
        u"皆",
        u"侦",
        u"悬",
        u"掘",
        u"享",
        u"纠",
        u"醒",
        u"狂",
        u"锁",
        u"淀",
        u"恨",
        u"牲",
        u"霸",
        u"爬",
        u"赏",
        u"逆",
        u"玩",
        u"陵",
        u"祝",
        u"秒",
        u"浙",
        u"貌",
    ]



# ed25519.py - Optimized version of the reference implementation of Ed25519
#
# Written in 2011? by Daniel J. Bernstein <djb@cr.yp.to>
#            2013 by Donald Stufft <donald@stufft.io>
#            2013 by Alex Gaynor <alex.gaynor@gmail.com>
#            2013 by Greg Price <price@mit.edu>
#            2019 by Michal Salaban <michal@salaban.info>
#
# To the extent possible under law, the author(s) have dedicated all copyright
# and related and neighboring rights to this software to the public domain
# worldwide. This software is distributed without any warranty.
#
# You should have received a copy of the CC0 Public Domain Dedication along
# with this software. If not, see
# <http://creativecommons.org/publicdomain/zero/1.0/>.

"""
NB: This code is not safe for use with secret keys or secret data.
The only safe use of this code is for verifying signatures on public messages.

Functions for computing the public key of a secret key and for signing
a message are included, namely publickey_unsafe and signature_unsafe,
for testing purposes only.

The root of the problem is that Python's long-integer arithmetic is
not designed for use in cryptography.  Specifically, it may take more
or less time to execute an operation depending on the values of the
inputs, and its memory access patterns may also depend on the inputs.
This opens it to timing and cache side-channel attacks which can
disclose data to an attacker.  We rely on Python's long-integer
arithmetic, so we cannot handle secrets without risking their disclosure.
"""

import binascii
import six
import sys


if sys.version_info >= (3,):  # pragma: no cover
    intlist2bytes = bytes
else:  # pragma: no cover
    range = xrange

    def intlist2bytes(l):
        return b"".join(chr(c) for c in l)


b = 256
q = 2 ** 255 - 19
l = 2 ** 252 + 27742317777372353535851937790883648493


def pow2(x, p):
    """== pow(x, 2**p, q)"""
    while p > 0:
        x = x * x % q
        p -= 1
    return x


def inv(z):
    # Adapted from curve25519_athlon.c in djb's Curve25519.
    z2 = z * z % q  # 2
    z9 = pow2(z2, 2) * z % q  # 9
    z11 = z9 * z2 % q  # 11
    z2_5_0 = (z11 * z11) % q * z9 % q  # 31 == 2^5 - 2^0
    z2_10_0 = pow2(z2_5_0, 5) * z2_5_0 % q  # 2^10 - 2^0
    z2_20_0 = pow2(z2_10_0, 10) * z2_10_0 % q  # ...
    z2_40_0 = pow2(z2_20_0, 20) * z2_20_0 % q
    z2_50_0 = pow2(z2_40_0, 10) * z2_10_0 % q
    z2_100_0 = pow2(z2_50_0, 50) * z2_50_0 % q
    z2_200_0 = pow2(z2_100_0, 100) * z2_100_0 % q
    z2_250_0 = pow2(z2_200_0, 50) * z2_50_0 % q  # 2^250 - 2^0
    return pow2(z2_250_0, 5) * z11 % q  # 2^255 - 2^5 + 11 = q - 2


d = -121665 * inv(121666) % q
I = pow(2, (q - 1) // 4, q)


def xrecover(y):
    xx = (y * y - 1) * inv(d * y * y + 1)
    x = pow(xx, (q + 3) // 8, q)

    if (x * x - xx) % q != 0:
        x = (x * I) % q

    if x % 2 != 0:
        x = q - x

    return x


def compress(P):
    zinv = inv(P[2])
    return (P[0] * zinv % q, P[1] * zinv % q)


def decompress(P):
    return (P[0], P[1], 1, P[0] * P[1] % q)


By = 4 * inv(5)
Bx = xrecover(By)
B = (Bx % q, By % q, 1, (Bx * By) % q)
ident = (0, 1, 1, 0)


def edwards_add(P, Q):
    # This is formula sequence 'addition-add-2008-hwcd-3' from
    # http://www.hyperelliptic.org/EFD/g1p/auto-twisted-extended-1.html
    (x1, y1, z1, t1) = P
    (x2, y2, z2, t2) = Q

    a = (y1 - x1) * (y2 - x2) % q
    b = (y1 + x1) * (y2 + x2) % q
    c = t1 * 2 * d * t2 % q
    dd = z1 * 2 * z2 % q
    e = b - a
    f = dd - c
    g = dd + c
    h = b + a
    x3 = e * f
    y3 = g * h
    t3 = e * h
    z3 = f * g

    return (x3 % q, y3 % q, z3 % q, t3 % q)


def edwards_double(P):
    # This is formula sequence 'dbl-2008-hwcd' from
    # http://www.hyperelliptic.org/EFD/g1p/auto-twisted-extended-1.html
    (x1, y1, z1, t1) = P

    a = x1 * x1 % q
    b = y1 * y1 % q
    c = 2 * z1 * z1 % q
    # dd = -a
    e = ((x1 + y1) * (x1 + y1) - a - b) % q
    g = -a + b  # dd + b
    f = g - c
    h = -a - b  # dd - b
    x3 = e * f
    y3 = g * h
    t3 = e * h
    z3 = f * g

    return (x3 % q, y3 % q, z3 % q, t3 % q)


def scalarmult(P, e):
    if e == 0:
        return ident
    Q = scalarmult(P, e // 2)
    Q = edwards_double(Q)
    if e & 1:
        Q = edwards_add(Q, P)
    return Q


# Bpow[i] == scalarmult(B, 2**i)
Bpow = []


def make_Bpow():
    P = B
    for i in range(253):
        Bpow.append(P)
        P = edwards_double(P)


make_Bpow()


def scalarmult_B(e):
    """
    Implements scalarmult(B, e) more efficiently.
    """
    # scalarmult(B, l) is the identity
    e = e % l
    P = ident
    for i in range(253):
        if e & 1:
            P = edwards_add(P, Bpow[i])
        e = e // 2
    assert e == 0, e
    return P


def encodeint(y):
    bits = [(y >> i) & 1 for i in range(b)]
    return b"".join(
        [
            six.int2byte(sum([bits[i * 8 + j] << j for j in range(8)]))
            for i in range(b // 8)
        ]
    )


def encodepoint(P):
    (x, y, z, t) = P
    zi = inv(z)
    x = (x * zi) % q
    y = (y * zi) % q
    bits = [(y >> i) & 1 for i in range(b - 1)] + [x & 1]
    return b"".join(
        [
            six.int2byte(sum([bits[i * 8 + j] << j for j in range(8)]))
            for i in range(b // 8)
        ]
    )


def bit(h, i):
    return (six.indexbytes(h, i // 8) >> (i % 8)) & 1


def isoncurve(P):
    (x, y, z, t) = P
    return (
        z % q != 0
        and x * y % q == z * t % q
        and (y * y - x * x - z * z - d * t * t) % q == 0
    )


def decodeint(s):
    return sum(2 ** i * bit(s, i) for i in range(0, b))


def decodepoint(s):
    y = sum(2 ** i * bit(s, i) for i in range(0, b - 1))
    x = xrecover(y)
    if x & 1 != bit(s, b - 1):
        x = q - x
    P = (x, y, 1, (x * y) % q)
    if not isoncurve(P):
        raise ValueError("decoding point that is not on curve")
    return P


def public_from_secret(k):
    keyInt = decodeint(k)
    aB = scalarmult_B(keyInt)
    return encodepoint(aB)


def public_from_secret_hex(hk):
    return binascii.hexlify(public_from_secret(binascii.unhexlify(hk))).decode()



cd_keccak = None
sha3_keccak = None

try:
    from Cryptodome.Hash import keccak as cd_keccak
except ImportError:
    pass
if not cd_keccak:
    try:
        from sha3 import keccak_256 as sha3_keccak
    except ImportError:
        raise ImportError(
            "Could not import 'keccak' from 'Cryptodome.Hash' nor 'keccak_256' from 'sha3'. "
            "Install one of those modules ('cryptodomex' is the recommended one)."
        )


def keccak_256(data):
    """
    Return a hashlib-compatible Keccak 256 object for the given data.
    """
    if cd_keccak is not None:
        h = cd_keccak.new(digest_bits=256)
        h.update(data)
    elif sha3_keccak is not None:
        h = sha3_keccak(data)
    else:  # pragma: no cover
        raise RuntimeError(
            "SHA3 implementation is missing. Install either 'pycryptodomex' (recommended) or 'pysha3' package to provide it"
        )
    return h



#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# Word list originally created by dabura667 and released under The MIT License (MIT)
#
# The MIT License (MIT)
#
# Permission is hereby granted, free of charge, to any person obtaining
# a copy of this software and associated documentation files (the
# "Software"), to deal in the Software without restriction, including
# without limitation the rights to use, copy, modify, merge, publish,
# distribute, sublicense, and/or sell copies of the Software, and to
# permit persons to whom the Software is furnished to do so, subject to
# the following conditions:
#
# The above copyright notice and this permission notice shall be
# included in all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
# EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
# MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
# NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE
# LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION
# OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION
# WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
#
# Code surrounding the word list is Copyright (c) 2014-2018, The Monero Project
#
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without modification, are
# permitted provided that the following conditions are met:
#
# 1. Redistributions of source code must retain the above copyright notice, this list of
#    conditions and the following disclaimer.
#
# 2. Redistributions in binary form must reproduce the above copyright notice, this list
#    of conditions and the following disclaimer in the documentation and/or other
#    materials provided with the distribution.
#
# 3. Neither the name of the copyright holder nor the names of its contributors may be
#    used to endorse or promote products derived from this software without specific
#    prior written permission.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY
# EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
# MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL
# THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
# SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
# PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
# INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
# STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF
# THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
#
# list of words from:
#     https://github.com/monero-project/monero/blob/master/src/mnemonics/spanish.h
#     Most recent commit: 8ea3c4d54429976e9001b8cc4868b721e6747e3c


from .wordlist import Wordlist


class Spanish(Wordlist):
    language_name = u"Español"
    english_language_name = "Spanish"
    unique_prefix_length = 4
    word_list = [
        u"ábaco",
        u"abdomen",
        u"abeja",
        u"abierto",
        u"abogado",
        u"abono",
        u"aborto",
        u"abrazo",
        u"abrir",
        u"abuelo",
        u"abuso",
        u"acabar",
        u"academia",
        u"acceso",
        u"acción",
        u"aceite",
        u"acelga",
        u"acento",
        u"aceptar",
        u"ácido",
        u"aclarar",
        u"acné",
        u"acoger",
        u"acoso",
        u"activo",
        u"acto",
        u"actriz",
        u"actuar",
        u"acudir",
        u"acuerdo",
        u"acusar",
        u"adicto",
        u"admitir",
        u"adoptar",
        u"adorno",
        u"aduana",
        u"adulto",
        u"aéreo",
        u"afectar",
        u"afición",
        u"afinar",
        u"afirmar",
        u"ágil",
        u"agitar",
        u"agonía",
        u"agosto",
        u"agotar",
        u"agregar",
        u"agrio",
        u"agua",
        u"agudo",
        u"águila",
        u"aguja",
        u"ahogo",
        u"ahorro",
        u"aire",
        u"aislar",
        u"ajedrez",
        u"ajeno",
        u"ajuste",
        u"alacrán",
        u"alambre",
        u"alarma",
        u"alba",
        u"álbum",
        u"alcalde",
        u"aldea",
        u"alegre",
        u"alejar",
        u"alerta",
        u"aleta",
        u"alfiler",
        u"alga",
        u"algodón",
        u"aliado",
        u"aliento",
        u"alivio",
        u"alma",
        u"almeja",
        u"almíbar",
        u"altar",
        u"alteza",
        u"altivo",
        u"alto",
        u"altura",
        u"alumno",
        u"alzar",
        u"amable",
        u"amante",
        u"amapola",
        u"amargo",
        u"amasar",
        u"ámbar",
        u"ámbito",
        u"ameno",
        u"amigo",
        u"amistad",
        u"amor",
        u"amparo",
        u"amplio",
        u"ancho",
        u"anciano",
        u"ancla",
        u"andar",
        u"andén",
        u"anemia",
        u"ángulo",
        u"anillo",
        u"ánimo",
        u"anís",
        u"anotar",
        u"antena",
        u"antiguo",
        u"antojo",
        u"anual",
        u"anular",
        u"anuncio",
        u"añadir",
        u"añejo",
        u"año",
        u"apagar",
        u"aparato",
        u"apetito",
        u"apio",
        u"aplicar",
        u"apodo",
        u"aporte",
        u"apoyo",
        u"aprender",
        u"aprobar",
        u"apuesta",
        u"apuro",
        u"arado",
        u"araña",
        u"arar",
        u"árbitro",
        u"árbol",
        u"arbusto",
        u"archivo",
        u"arco",
        u"arder",
        u"ardilla",
        u"arduo",
        u"área",
        u"árido",
        u"aries",
        u"armonía",
        u"arnés",
        u"aroma",
        u"arpa",
        u"arpón",
        u"arreglo",
        u"arroz",
        u"arruga",
        u"arte",
        u"artista",
        u"asa",
        u"asado",
        u"asalto",
        u"ascenso",
        u"asegurar",
        u"aseo",
        u"asesor",
        u"asiento",
        u"asilo",
        u"asistir",
        u"asno",
        u"asombro",
        u"áspero",
        u"astilla",
        u"astro",
        u"astuto",
        u"asumir",
        u"asunto",
        u"atajo",
        u"ataque",
        u"atar",
        u"atento",
        u"ateo",
        u"ático",
        u"atleta",
        u"átomo",
        u"atraer",
        u"atroz",
        u"atún",
        u"audaz",
        u"audio",
        u"auge",
        u"aula",
        u"aumento",
        u"ausente",
        u"autor",
        u"aval",
        u"avance",
        u"avaro",
        u"ave",
        u"avellana",
        u"avena",
        u"avestruz",
        u"avión",
        u"aviso",
        u"ayer",
        u"ayuda",
        u"ayuno",
        u"azafrán",
        u"azar",
        u"azote",
        u"azúcar",
        u"azufre",
        u"azul",
        u"baba",
        u"babor",
        u"bache",
        u"bahía",
        u"baile",
        u"bajar",
        u"balanza",
        u"balcón",
        u"balde",
        u"bambú",
        u"banco",
        u"banda",
        u"baño",
        u"barba",
        u"barco",
        u"barniz",
        u"barro",
        u"báscula",
        u"bastón",
        u"basura",
        u"batalla",
        u"batería",
        u"batir",
        u"batuta",
        u"baúl",
        u"bazar",
        u"bebé",
        u"bebida",
        u"bello",
        u"besar",
        u"beso",
        u"bestia",
        u"bicho",
        u"bien",
        u"bingo",
        u"blanco",
        u"bloque",
        u"blusa",
        u"boa",
        u"bobina",
        u"bobo",
        u"boca",
        u"bocina",
        u"boda",
        u"bodega",
        u"boina",
        u"bola",
        u"bolero",
        u"bolsa",
        u"bomba",
        u"bondad",
        u"bonito",
        u"bono",
        u"bonsái",
        u"borde",
        u"borrar",
        u"bosque",
        u"bote",
        u"botín",
        u"bóveda",
        u"bozal",
        u"bravo",
        u"brazo",
        u"brecha",
        u"breve",
        u"brillo",
        u"brinco",
        u"brisa",
        u"broca",
        u"broma",
        u"bronce",
        u"brote",
        u"bruja",
        u"brusco",
        u"bruto",
        u"buceo",
        u"bucle",
        u"bueno",
        u"buey",
        u"bufanda",
        u"bufón",
        u"búho",
        u"buitre",
        u"bulto",
        u"burbuja",
        u"burla",
        u"burro",
        u"buscar",
        u"butaca",
        u"buzón",
        u"caballo",
        u"cabeza",
        u"cabina",
        u"cabra",
        u"cacao",
        u"cadáver",
        u"cadena",
        u"caer",
        u"café",
        u"caída",
        u"caimán",
        u"caja",
        u"cajón",
        u"cal",
        u"calamar",
        u"calcio",
        u"caldo",
        u"calidad",
        u"calle",
        u"calma",
        u"calor",
        u"calvo",
        u"cama",
        u"cambio",
        u"camello",
        u"camino",
        u"campo",
        u"cáncer",
        u"candil",
        u"canela",
        u"canguro",
        u"canica",
        u"canto",
        u"caña",
        u"cañón",
        u"caoba",
        u"caos",
        u"capaz",
        u"capitán",
        u"capote",
        u"captar",
        u"capucha",
        u"cara",
        u"carbón",
        u"cárcel",
        u"careta",
        u"carga",
        u"cariño",
        u"carne",
        u"carpeta",
        u"carro",
        u"carta",
        u"casa",
        u"casco",
        u"casero",
        u"caspa",
        u"castor",
        u"catorce",
        u"catre",
        u"caudal",
        u"causa",
        u"cazo",
        u"cebolla",
        u"ceder",
        u"cedro",
        u"celda",
        u"célebre",
        u"celoso",
        u"célula",
        u"cemento",
        u"ceniza",
        u"centro",
        u"cerca",
        u"cerdo",
        u"cereza",
        u"cero",
        u"cerrar",
        u"certeza",
        u"césped",
        u"cetro",
        u"chacal",
        u"chaleco",
        u"champú",
        u"chancla",
        u"chapa",
        u"charla",
        u"chico",
        u"chiste",
        u"chivo",
        u"choque",
        u"choza",
        u"chuleta",
        u"chupar",
        u"ciclón",
        u"ciego",
        u"cielo",
        u"cien",
        u"cierto",
        u"cifra",
        u"cigarro",
        u"cima",
        u"cinco",
        u"cine",
        u"cinta",
        u"ciprés",
        u"circo",
        u"ciruela",
        u"cisne",
        u"cita",
        u"ciudad",
        u"clamor",
        u"clan",
        u"claro",
        u"clase",
        u"clave",
        u"cliente",
        u"clima",
        u"clínica",
        u"cobre",
        u"cocción",
        u"cochino",
        u"cocina",
        u"coco",
        u"código",
        u"codo",
        u"cofre",
        u"coger",
        u"cohete",
        u"cojín",
        u"cojo",
        u"cola",
        u"colcha",
        u"colegio",
        u"colgar",
        u"colina",
        u"collar",
        u"colmo",
        u"columna",
        u"combate",
        u"comer",
        u"comida",
        u"cómodo",
        u"compra",
        u"conde",
        u"conejo",
        u"conga",
        u"conocer",
        u"consejo",
        u"contar",
        u"copa",
        u"copia",
        u"corazón",
        u"corbata",
        u"corcho",
        u"cordón",
        u"corona",
        u"correr",
        u"coser",
        u"cosmos",
        u"costa",
        u"cráneo",
        u"cráter",
        u"crear",
        u"crecer",
        u"creído",
        u"crema",
        u"cría",
        u"crimen",
        u"cripta",
        u"crisis",
        u"cromo",
        u"crónica",
        u"croqueta",
        u"crudo",
        u"cruz",
        u"cuadro",
        u"cuarto",
        u"cuatro",
        u"cubo",
        u"cubrir",
        u"cuchara",
        u"cuello",
        u"cuento",
        u"cuerda",
        u"cuesta",
        u"cueva",
        u"cuidar",
        u"culebra",
        u"culpa",
        u"culto",
        u"cumbre",
        u"cumplir",
        u"cuna",
        u"cuneta",
        u"cuota",
        u"cupón",
        u"cúpula",
        u"curar",
        u"curioso",
        u"curso",
        u"curva",
        u"cutis",
        u"dama",
        u"danza",
        u"dar",
        u"dardo",
        u"dátil",
        u"deber",
        u"débil",
        u"década",
        u"decir",
        u"dedo",
        u"defensa",
        u"definir",
        u"dejar",
        u"delfín",
        u"delgado",
        u"delito",
        u"demora",
        u"denso",
        u"dental",
        u"deporte",
        u"derecho",
        u"derrota",
        u"desayuno",
        u"deseo",
        u"desfile",
        u"desnudo",
        u"destino",
        u"desvío",
        u"detalle",
        u"detener",
        u"deuda",
        u"día",
        u"diablo",
        u"diadema",
        u"diamante",
        u"diana",
        u"diario",
        u"dibujo",
        u"dictar",
        u"diente",
        u"dieta",
        u"diez",
        u"difícil",
        u"digno",
        u"dilema",
        u"diluir",
        u"dinero",
        u"directo",
        u"dirigir",
        u"disco",
        u"diseño",
        u"disfraz",
        u"diva",
        u"divino",
        u"doble",
        u"doce",
        u"dolor",
        u"domingo",
        u"don",
        u"donar",
        u"dorado",
        u"dormir",
        u"dorso",
        u"dos",
        u"dosis",
        u"dragón",
        u"droga",
        u"ducha",
        u"duda",
        u"duelo",
        u"dueño",
        u"dulce",
        u"dúo",
        u"duque",
        u"durar",
        u"dureza",
        u"duro",
        u"ébano",
        u"ebrio",
        u"echar",
        u"eco",
        u"ecuador",
        u"edad",
        u"edición",
        u"edificio",
        u"editor",
        u"educar",
        u"efecto",
        u"eficaz",
        u"eje",
        u"ejemplo",
        u"elefante",
        u"elegir",
        u"elemento",
        u"elevar",
        u"elipse",
        u"élite",
        u"elixir",
        u"elogio",
        u"eludir",
        u"embudo",
        u"emitir",
        u"emoción",
        u"empate",
        u"empeño",
        u"empleo",
        u"empresa",
        u"enano",
        u"encargo",
        u"enchufe",
        u"encía",
        u"enemigo",
        u"enero",
        u"enfado",
        u"enfermo",
        u"engaño",
        u"enigma",
        u"enlace",
        u"enorme",
        u"enredo",
        u"ensayo",
        u"enseñar",
        u"entero",
        u"entrar",
        u"envase",
        u"envío",
        u"época",
        u"equipo",
        u"erizo",
        u"escala",
        u"escena",
        u"escolar",
        u"escribir",
        u"escudo",
        u"esencia",
        u"esfera",
        u"esfuerzo",
        u"espada",
        u"espejo",
        u"espía",
        u"esposa",
        u"espuma",
        u"esquí",
        u"estar",
        u"este",
        u"estilo",
        u"estufa",
        u"etapa",
        u"eterno",
        u"ética",
        u"etnia",
        u"evadir",
        u"evaluar",
        u"evento",
        u"evitar",
        u"exacto",
        u"examen",
        u"exceso",
        u"excusa",
        u"exento",
        u"exigir",
        u"exilio",
        u"existir",
        u"éxito",
        u"experto",
        u"explicar",
        u"exponer",
        u"extremo",
        u"fábrica",
        u"fábula",
        u"fachada",
        u"fácil",
        u"factor",
        u"faena",
        u"faja",
        u"falda",
        u"fallo",
        u"falso",
        u"faltar",
        u"fama",
        u"familia",
        u"famoso",
        u"faraón",
        u"farmacia",
        u"farol",
        u"farsa",
        u"fase",
        u"fatiga",
        u"fauna",
        u"favor",
        u"fax",
        u"febrero",
        u"fecha",
        u"feliz",
        u"feo",
        u"feria",
        u"feroz",
        u"fértil",
        u"fervor",
        u"festín",
        u"fiable",
        u"fianza",
        u"fiar",
        u"fibra",
        u"ficción",
        u"ficha",
        u"fideo",
        u"fiebre",
        u"fiel",
        u"fiera",
        u"fiesta",
        u"figura",
        u"fijar",
        u"fijo",
        u"fila",
        u"filete",
        u"filial",
        u"filtro",
        u"fin",
        u"finca",
        u"fingir",
        u"finito",
        u"firma",
        u"flaco",
        u"flauta",
        u"flecha",
        u"flor",
        u"flota",
        u"fluir",
        u"flujo",
        u"flúor",
        u"fobia",
        u"foca",
        u"fogata",
        u"fogón",
        u"folio",
        u"folleto",
        u"fondo",
        u"forma",
        u"forro",
        u"fortuna",
        u"forzar",
        u"fosa",
        u"foto",
        u"fracaso",
        u"frágil",
        u"franja",
        u"frase",
        u"fraude",
        u"freír",
        u"freno",
        u"fresa",
        u"frío",
        u"frito",
        u"fruta",
        u"fuego",
        u"fuente",
        u"fuerza",
        u"fuga",
        u"fumar",
        u"función",
        u"funda",
        u"furgón",
        u"furia",
        u"fusil",
        u"fútbol",
        u"futuro",
        u"gacela",
        u"gafas",
        u"gaita",
        u"gajo",
        u"gala",
        u"galería",
        u"gallo",
        u"gamba",
        u"ganar",
        u"gancho",
        u"ganga",
        u"ganso",
        u"garaje",
        u"garza",
        u"gasolina",
        u"gastar",
        u"gato",
        u"gavilán",
        u"gemelo",
        u"gemir",
        u"gen",
        u"género",
        u"genio",
        u"gente",
        u"geranio",
        u"gerente",
        u"germen",
        u"gesto",
        u"gigante",
        u"gimnasio",
        u"girar",
        u"giro",
        u"glaciar",
        u"globo",
        u"gloria",
        u"gol",
        u"golfo",
        u"goloso",
        u"golpe",
        u"goma",
        u"gordo",
        u"gorila",
        u"gorra",
        u"gota",
        u"goteo",
        u"gozar",
        u"grada",
        u"gráfico",
        u"grano",
        u"grasa",
        u"gratis",
        u"grave",
        u"grieta",
        u"grillo",
        u"gripe",
        u"gris",
        u"grito",
        u"grosor",
        u"grúa",
        u"grueso",
        u"grumo",
        u"grupo",
        u"guante",
        u"guapo",
        u"guardia",
        u"guerra",
        u"guía",
        u"guiño",
        u"guion",
        u"guiso",
        u"guitarra",
        u"gusano",
        u"gustar",
        u"haber",
        u"hábil",
        u"hablar",
        u"hacer",
        u"hacha",
        u"hada",
        u"hallar",
        u"hamaca",
        u"harina",
        u"haz",
        u"hazaña",
        u"hebilla",
        u"hebra",
        u"hecho",
        u"helado",
        u"helio",
        u"hembra",
        u"herir",
        u"hermano",
        u"héroe",
        u"hervir",
        u"hielo",
        u"hierro",
        u"hígado",
        u"higiene",
        u"hijo",
        u"himno",
        u"historia",
        u"hocico",
        u"hogar",
        u"hoguera",
        u"hoja",
        u"hombre",
        u"hongo",
        u"honor",
        u"honra",
        u"hora",
        u"hormiga",
        u"horno",
        u"hostil",
        u"hoyo",
        u"hueco",
        u"huelga",
        u"huerta",
        u"hueso",
        u"huevo",
        u"huida",
        u"huir",
        u"humano",
        u"húmedo",
        u"humilde",
        u"humo",
        u"hundir",
        u"huracán",
        u"hurto",
        u"icono",
        u"ideal",
        u"idioma",
        u"ídolo",
        u"iglesia",
        u"iglú",
        u"igual",
        u"ilegal",
        u"ilusión",
        u"imagen",
        u"imán",
        u"imitar",
        u"impar",
        u"imperio",
        u"imponer",
        u"impulso",
        u"incapaz",
        u"índice",
        u"inerte",
        u"infiel",
        u"informe",
        u"ingenio",
        u"inicio",
        u"inmenso",
        u"inmune",
        u"innato",
        u"insecto",
        u"instante",
        u"interés",
        u"íntimo",
        u"intuir",
        u"inútil",
        u"invierno",
        u"ira",
        u"iris",
        u"ironía",
        u"isla",
        u"islote",
        u"jabalí",
        u"jabón",
        u"jamón",
        u"jarabe",
        u"jardín",
        u"jarra",
        u"jaula",
        u"jazmín",
        u"jefe",
        u"jeringa",
        u"jinete",
        u"jornada",
        u"joroba",
        u"joven",
        u"joya",
        u"juerga",
        u"jueves",
        u"juez",
        u"jugador",
        u"jugo",
        u"juguete",
        u"juicio",
        u"junco",
        u"jungla",
        u"junio",
        u"juntar",
        u"júpiter",
        u"jurar",
        u"justo",
        u"juvenil",
        u"juzgar",
        u"kilo",
        u"koala",
        u"labio",
        u"lacio",
        u"lacra",
        u"lado",
        u"ladrón",
        u"lagarto",
        u"lágrima",
        u"laguna",
        u"laico",
        u"lamer",
        u"lámina",
        u"lámpara",
        u"lana",
        u"lancha",
        u"langosta",
        u"lanza",
        u"lápiz",
        u"largo",
        u"larva",
        u"lástima",
        u"lata",
        u"látex",
        u"latir",
        u"laurel",
        u"lavar",
        u"lazo",
        u"leal",
        u"lección",
        u"leche",
        u"lector",
        u"leer",
        u"legión",
        u"legumbre",
        u"lejano",
        u"lengua",
        u"lento",
        u"leña",
        u"león",
        u"leopardo",
        u"lesión",
        u"letal",
        u"letra",
        u"leve",
        u"leyenda",
        u"libertad",
        u"libro",
        u"licor",
        u"líder",
        u"lidiar",
        u"lienzo",
        u"liga",
        u"ligero",
        u"lima",
        u"límite",
        u"limón",
        u"limpio",
        u"lince",
        u"lindo",
        u"línea",
        u"lingote",
        u"lino",
        u"linterna",
        u"líquido",
        u"liso",
        u"lista",
        u"litera",
        u"litio",
        u"litro",
        u"llaga",
        u"llama",
        u"llanto",
        u"llave",
        u"llegar",
        u"llenar",
        u"llevar",
        u"llorar",
        u"llover",
        u"lluvia",
        u"lobo",
        u"loción",
        u"loco",
        u"locura",
        u"lógica",
        u"logro",
        u"lombriz",
        u"lomo",
        u"lonja",
        u"lote",
        u"lucha",
        u"lucir",
        u"lugar",
        u"lujo",
        u"luna",
        u"lunes",
        u"lupa",
        u"lustro",
        u"luto",
        u"luz",
        u"maceta",
        u"macho",
        u"madera",
        u"madre",
        u"maduro",
        u"maestro",
        u"mafia",
        u"magia",
        u"mago",
        u"maíz",
        u"maldad",
        u"maleta",
        u"malla",
        u"malo",
        u"mamá",
        u"mambo",
        u"mamut",
        u"manco",
        u"mando",
        u"manejar",
        u"manga",
        u"maniquí",
        u"manjar",
        u"mano",
        u"manso",
        u"manta",
        u"mañana",
        u"mapa",
        u"máquina",
        u"mar",
        u"marco",
        u"marea",
        u"marfil",
        u"margen",
        u"marido",
        u"mármol",
        u"marrón",
        u"martes",
        u"marzo",
        u"masa",
        u"máscara",
        u"masivo",
        u"matar",
        u"materia",
        u"matiz",
        u"matriz",
        u"máximo",
        u"mayor",
        u"mazorca",
        u"mecha",
        u"medalla",
        u"medio",
        u"médula",
        u"mejilla",
        u"mejor",
        u"melena",
        u"melón",
        u"memoria",
        u"menor",
        u"mensaje",
        u"mente",
        u"menú",
        u"mercado",
        u"merengue",
        u"mérito",
        u"mes",
        u"mesón",
        u"meta",
        u"meter",
        u"método",
        u"metro",
        u"mezcla",
        u"miedo",
        u"miel",
        u"miembro",
        u"miga",
        u"mil",
        u"milagro",
        u"militar",
        u"millón",
        u"mimo",
        u"mina",
        u"minero",
        u"mínimo",
        u"minuto",
        u"miope",
        u"mirar",
        u"misa",
        u"miseria",
        u"misil",
        u"mismo",
        u"mitad",
        u"mito",
        u"mochila",
        u"moción",
        u"moda",
        u"modelo",
        u"moho",
        u"mojar",
        u"molde",
        u"moler",
        u"molino",
        u"momento",
        u"momia",
        u"monarca",
        u"moneda",
        u"monja",
        u"monto",
        u"moño",
        u"morada",
        u"morder",
        u"moreno",
        u"morir",
        u"morro",
        u"morsa",
        u"mortal",
        u"mosca",
        u"mostrar",
        u"motivo",
        u"mover",
        u"móvil",
        u"mozo",
        u"mucho",
        u"mudar",
        u"mueble",
        u"muela",
        u"muerte",
        u"muestra",
        u"mugre",
        u"mujer",
        u"mula",
        u"muleta",
        u"multa",
        u"mundo",
        u"muñeca",
        u"mural",
        u"muro",
        u"músculo",
        u"museo",
        u"musgo",
        u"música",
        u"muslo",
        u"nácar",
        u"nación",
        u"nadar",
        u"naipe",
        u"naranja",
        u"nariz",
        u"narrar",
        u"nasal",
        u"natal",
        u"nativo",
        u"natural",
        u"náusea",
        u"naval",
        u"nave",
        u"navidad",
        u"necio",
        u"néctar",
        u"negar",
        u"negocio",
        u"negro",
        u"neón",
        u"nervio",
        u"neto",
        u"neutro",
        u"nevar",
        u"nevera",
        u"nicho",
        u"nido",
        u"niebla",
        u"nieto",
        u"niñez",
        u"niño",
        u"nítido",
        u"nivel",
        u"nobleza",
        u"noche",
        u"nómina",
        u"noria",
        u"norma",
        u"norte",
        u"nota",
        u"noticia",
        u"novato",
        u"novela",
        u"novio",
        u"nube",
        u"nuca",
        u"núcleo",
        u"nudillo",
        u"nudo",
        u"nuera",
        u"nueve",
        u"nuez",
        u"nulo",
        u"número",
        u"nutria",
        u"oasis",
        u"obeso",
        u"obispo",
        u"objeto",
        u"obra",
        u"obrero",
        u"observar",
        u"obtener",
        u"obvio",
        u"oca",
        u"ocaso",
        u"océano",
        u"ochenta",
        u"ocho",
        u"ocio",
        u"ocre",
        u"octavo",
        u"octubre",
        u"oculto",
        u"ocupar",
        u"ocurrir",
        u"odiar",
        u"odio",
        u"odisea",
        u"oeste",
        u"ofensa",
        u"oferta",
        u"oficio",
        u"ofrecer",
        u"ogro",
        u"oído",
        u"oír",
        u"ojo",
        u"ola",
        u"oleada",
        u"olfato",
        u"olivo",
        u"olla",
        u"olmo",
        u"olor",
        u"olvido",
        u"ombligo",
        u"onda",
        u"onza",
        u"opaco",
        u"opción",
        u"ópera",
        u"opinar",
        u"oponer",
        u"optar",
        u"óptica",
        u"opuesto",
        u"oración",
        u"orador",
        u"oral",
        u"órbita",
        u"orca",
        u"orden",
        u"oreja",
        u"órgano",
        u"orgía",
        u"orgullo",
        u"oriente",
        u"origen",
        u"orilla",
        u"oro",
        u"orquesta",
        u"oruga",
        u"osadía",
        u"oscuro",
        u"osezno",
        u"oso",
        u"ostra",
        u"otoño",
        u"otro",
        u"oveja",
        u"óvulo",
        u"óxido",
        u"oxígeno",
        u"oyente",
        u"ozono",
        u"pacto",
        u"padre",
        u"paella",
        u"página",
        u"pago",
        u"país",
        u"pájaro",
        u"palabra",
        u"palco",
        u"paleta",
        u"pálido",
        u"palma",
        u"paloma",
        u"palpar",
        u"pan",
        u"panal",
        u"pánico",
        u"pantera",
        u"pañuelo",
        u"papá",
        u"papel",
        u"papilla",
        u"paquete",
        u"parar",
        u"parcela",
        u"pared",
        u"parir",
        u"paro",
        u"párpado",
        u"parque",
        u"párrafo",
        u"parte",
        u"pasar",
        u"paseo",
        u"pasión",
        u"paso",
        u"pasta",
        u"pata",
        u"patio",
        u"patria",
        u"pausa",
        u"pauta",
        u"pavo",
        u"payaso",
        u"peatón",
        u"pecado",
        u"pecera",
        u"pecho",
        u"pedal",
        u"pedir",
        u"pegar",
        u"peine",
        u"pelar",
        u"peldaño",
        u"pelea",
        u"peligro",
        u"pellejo",
        u"pelo",
        u"peluca",
        u"pena",
        u"pensar",
        u"peñón",
        u"peón",
        u"peor",
        u"pepino",
        u"pequeño",
        u"pera",
        u"percha",
        u"perder",
        u"pereza",
        u"perfil",
        u"perico",
        u"perla",
        u"permiso",
        u"perro",
        u"persona",
        u"pesa",
        u"pesca",
        u"pésimo",
        u"pestaña",
        u"pétalo",
        u"petróleo",
        u"pez",
        u"pezuña",
        u"picar",
        u"pichón",
        u"pie",
        u"piedra",
        u"pierna",
        u"pieza",
        u"pijama",
        u"pilar",
        u"piloto",
        u"pimienta",
        u"pino",
        u"pintor",
        u"pinza",
        u"piña",
        u"piojo",
        u"pipa",
        u"pirata",
        u"pisar",
        u"piscina",
        u"piso",
        u"pista",
        u"pitón",
        u"pizca",
        u"placa",
        u"plan",
        u"plata",
        u"playa",
        u"plaza",
        u"pleito",
        u"pleno",
        u"plomo",
        u"pluma",
        u"plural",
        u"pobre",
        u"poco",
        u"poder",
        u"podio",
        u"poema",
        u"poesía",
        u"poeta",
        u"polen",
        u"policía",
        u"pollo",
        u"polvo",
        u"pomada",
        u"pomelo",
        u"pomo",
        u"pompa",
        u"poner",
        u"porción",
        u"portal",
        u"posada",
        u"poseer",
        u"posible",
        u"poste",
        u"potencia",
        u"potro",
        u"pozo",
        u"prado",
        u"precoz",
        u"pregunta",
        u"premio",
        u"prensa",
        u"preso",
        u"previo",
        u"primo",
        u"príncipe",
        u"prisión",
        u"privar",
        u"proa",
        u"probar",
        u"proceso",
        u"producto",
        u"proeza",
        u"profesor",
        u"programa",
        u"prole",
        u"promesa",
        u"pronto",
        u"propio",
        u"próximo",
        u"prueba",
        u"público",
        u"puchero",
        u"pudor",
        u"pueblo",
        u"puerta",
        u"puesto",
        u"pulga",
        u"pulir",
        u"pulmón",
        u"pulpo",
        u"pulso",
        u"puma",
        u"punto",
        u"puñal",
        u"puño",
        u"pupa",
        u"pupila",
        u"puré",
        u"quedar",
        u"queja",
        u"quemar",
        u"querer",
        u"queso",
        u"quieto",
        u"química",
        u"quince",
        u"quitar",
        u"rábano",
        u"rabia",
        u"rabo",
        u"ración",
        u"radical",
        u"raíz",
        u"rama",
        u"rampa",
        u"rancho",
        u"rango",
        u"rapaz",
        u"rápido",
        u"rapto",
        u"rasgo",
        u"raspa",
        u"rato",
        u"rayo",
        u"raza",
        u"razón",
        u"reacción",
        u"realidad",
        u"rebaño",
        u"rebote",
        u"recaer",
        u"receta",
        u"rechazo",
        u"recoger",
        u"recreo",
        u"recto",
        u"recurso",
        u"red",
        u"redondo",
        u"reducir",
        u"reflejo",
        u"reforma",
        u"refrán",
        u"refugio",
        u"regalo",
        u"regir",
        u"regla",
        u"regreso",
        u"rehén",
        u"reino",
        u"reír",
        u"reja",
        u"relato",
        u"relevo",
        u"relieve",
        u"relleno",
        u"reloj",
        u"remar",
        u"remedio",
        u"remo",
        u"rencor",
        u"rendir",
        u"renta",
        u"reparto",
        u"repetir",
        u"reposo",
        u"reptil",
        u"res",
        u"rescate",
        u"resina",
        u"respeto",
        u"resto",
        u"resumen",
        u"retiro",
        u"retorno",
        u"retrato",
        u"reunir",
        u"revés",
        u"revista",
        u"rey",
        u"rezar",
        u"rico",
        u"riego",
        u"rienda",
        u"riesgo",
        u"rifa",
        u"rígido",
        u"rigor",
        u"rincón",
        u"riñón",
        u"río",
        u"riqueza",
        u"risa",
        u"ritmo",
        u"rito",
    ]



import binascii
import six
import varint


class ExtraParser(object):
    TX_EXTRA_TAG_PADDING = 0x00
    TX_EXTRA_TAG_PUBKEY = 0x01
    TX_EXTRA_TAG_EXTRA_NONCE = 0x02
    TX_EXTRA_TAG_ADDITIONAL_PUBKEYS = 0x04
    KNOWN_TAGS = (
        TX_EXTRA_TAG_PADDING,
        TX_EXTRA_TAG_PUBKEY,
        TX_EXTRA_TAG_EXTRA_NONCE,
        TX_EXTRA_TAG_ADDITIONAL_PUBKEYS,
    )

    def __init__(self, extra):
        if isinstance(extra, str):
            extra = binascii.unhexlify(extra)
        if isinstance(extra, bytes):
            extra = list(extra)
        self.extra = extra

    def parse(self):
        self.data = {}
        self.offset = 0
        self._parse(self.extra)
        return self.data

    def _strip_padding(self, extra):
        while extra and extra[0] == self.TX_EXTRA_TAG_PADDING:
            extra = extra[1:]
            self.offset += 1
        return extra

    def _pop_pubkey(self, extra):
        key = bytes(bytearray(extra[:32]))  # bytearray() is for py2 compatibility
        if len(key) < 32:
            raise ValueError(
                "offset {:d}: only {:d} bytes of key data, expected 32".format(
                    self.offset, len(key)
                )
            )
        if "pubkeys" in self.data:
            self.data["pubkeys"].append(key)
        else:
            self.data["pubkeys"] = [key]
        extra = extra[32:]
        self.offset += 32
        return extra

    def _extract_pubkey(self, extra):
        if extra:
            if extra[0] == self.TX_EXTRA_TAG_PUBKEY:
                extra = extra[1:]
                self.offset += 1
                extra = self._pop_pubkey(extra)
            elif extra[0] == self.TX_EXTRA_TAG_ADDITIONAL_PUBKEYS:
                extra = extra[1:]
                self.offset += 1
                keycount = varint.decode_bytes(bytearray(extra))
                valen = len(varint.encode(keycount))
                extra = extra[valen:]
                self.offset += valen
                for i in range(keycount):
                    extra = self._pop_pubkey(extra)
        return extra

    def _extract_nonce(self, extra):
        if extra and extra[0] == self.TX_EXTRA_TAG_EXTRA_NONCE:
            noncelen = extra[1]
            extra = extra[2:]
            self.offset += 2
            if noncelen > len(extra):
                raise ValueError(
                    "offset {:d}: extra nonce exceeds field size".format(self.offset)
                )
                return []
            nonce = bytearray(extra[:noncelen])
            if "nonces" in self.data:
                self.data["nonces"].append(nonce)
            else:
                self.data["nonces"] = [nonce]
            extra = extra[noncelen:]
            self.offset += noncelen
        return extra

    def _parse(self, extra):
        while extra:
            if extra[0] not in self.KNOWN_TAGS:
                raise ValueError(
                    "offset {:d}: unknown tag 0x{:x}".format(self.offset, extra[0])
                )
            extra = self._strip_padding(extra)
            extra = self._extract_pubkey(extra)
            extra = self._extract_nonce(extra)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# Word list originally created by dabura667 and released under The MIT License (MIT)
#
# The MIT License (MIT)
#
# Permission is hereby granted, free of charge, to any person obtaining
# a copy of this software and associated documentation files (the
# "Software"), to deal in the Software without restriction, including
# without limitation the rights to use, copy, modify, merge, publish,
# distribute, sublicense, and/or sell copies of the Software, and to
# permit persons to whom the Software is furnished to do so, subject to
# the following conditions:
#
# The above copyright notice and this permission notice shall be
# included in all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
# EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
# MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
# NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE
# LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION
# OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION
# WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
#
# Code surrounding the word list is Copyright (c) 2014-2018, The Monero Project
#
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without modification, are
# permitted provided that the following conditions are met:
#
# 1. Redistributions of source code must retain the above copyright notice, this list of
#    conditions and the following disclaimer.
#
# 2. Redistributions in binary form must reproduce the above copyright notice, this list
#    of conditions and the following disclaimer in the documentation and/or other
#    materials provided with the distribution.
#
# 3. Neither the name of the copyright holder nor the names of its contributors may be
#    used to endorse or promote products derived from this software without specific
#    prior written permission.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY
# EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
# MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL
# THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
# SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
# PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
# INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
# STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF
# THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
#
# list of words from:
#     https://github.com/monero-project/monero/blob/master/src/mnemonics/french.h
#     Most recent commit: 8ea3c4d54429976e9001b8cc4868b721e6747e3c


from .wordlist import Wordlist


class French(Wordlist):
    language_name = u"Français"
    english_language_name = "French"
    unique_prefix_length = 4
    word_list = [
        "abandon",
        "abattre",
        "aboi",
        "abolir",
        "aborder",
        "abri",
        "absence",
        "absolu",
        "abuser",
        "acacia",
        "acajou",
        "accent",
        "accord",
        "accrocher",
        "accuser",
        "acerbe",
        "achat",
        "acheter",
        "acide",
        "acier",
        "acquis",
        "acte",
        "action",
        "adage",
        "adepte",
        "adieu",
        "admettre",
        "admis",
        "adorer",
        "adresser",
        "aduler",
        "affaire",
        "affirmer",
        "afin",
        "agacer",
        "agent",
        "agir",
        "agiter",
        "agonie",
        "agrafe",
        "agrume",
        "aider",
        "aigle",
        "aigre",
        "aile",
        "ailleurs",
        "aimant",
        "aimer",
        "ainsi",
        "aise",
        "ajouter",
        "alarme",
        "album",
        "alcool",
        "alerte",
        "algue",
        "alibi",
        "aller",
        "allumer",
        "alors",
        "amande",
        "amener",
        "amie",
        "amorcer",
        "amour",
        "ample",
        "amuser",
        "ananas",
        "ancien",
        "anglais",
        "angoisse",
        "animal",
        "anneau",
        "annoncer",
        "apercevoir",
        "apparence",
        "appel",
        "apporter",
        "apprendre",
        "appuyer",
        "arbre",
        "arcade",
        "arceau",
        "arche",
        "ardeur",
        "argent",
        "argile",
        "aride",
        "arme",
        "armure",
        "arracher",
        "arriver",
        "article",
        "asile",
        "aspect",
        "assaut",
        "assez",
        "assister",
        "assurer",
        "astre",
        "astuce",
        "atlas",
        "atroce",
        "attacher",
        "attente",
        "attirer",
        "aube",
        "aucun",
        "audace",
        "auparavant",
        "auquel",
        "aurore",
        "aussi",
        "autant",
        "auteur",
        "autoroute",
        "autre",
        "aval",
        "avant",
        "avec",
        "avenir",
        "averse",
        "aveu",
        "avide",
        "avion",
        "avis",
        "avoir",
        "avouer",
        "avril",
        "azote",
        "azur",
        "badge",
        "bagage",
        "bague",
        "bain",
        "baisser",
        "balai",
        "balcon",
        "balise",
        "balle",
        "bambou",
        "banane",
        "banc",
        "bandage",
        "banjo",
        "banlieue",
        "bannir",
        "banque",
        "baobab",
        "barbe",
        "barque",
        "barrer",
        "bassine",
        "bataille",
        "bateau",
        "battre",
        "baver",
        "bavoir",
        "bazar",
        "beau",
        "beige",
        "berger",
        "besoin",
        "beurre",
        "biais",
        "biceps",
        "bidule",
        "bien",
        "bijou",
        "bilan",
        "billet",
        "blanc",
        "blason",
        "bleu",
        "bloc",
        "blond",
        "bocal",
        "boire",
        "boiserie",
        "boiter",
        "bonbon",
        "bondir",
        "bonheur",
        "bordure",
        "borgne",
        "borner",
        "bosse",
        "bouche",
        "bouder",
        "bouger",
        "boule",
        "bourse",
        "bout",
        "boxe",
        "brader",
        "braise",
        "branche",
        "braquer",
        "bras",
        "brave",
        "brebis",
        "brevet",
        "brider",
        "briller",
        "brin",
        "brique",
        "briser",
        "broche",
        "broder",
        "bronze",
        "brosser",
        "brouter",
        "bruit",
        "brute",
        "budget",
        "buffet",
        "bulle",
        "bureau",
        "buriner",
        "buste",
        "buter",
        "butiner",
        "cabas",
        "cabinet",
        "cabri",
        "cacao",
        "cacher",
        "cadeau",
        "cadre",
        "cage",
        "caisse",
        "caler",
        "calme",
        "camarade",
        "camion",
        "campagne",
        "canal",
        "canif",
        "capable",
        "capot",
        "carat",
        "caresser",
        "carie",
        "carpe",
        "cartel",
        "casier",
        "casque",
        "casserole",
        "cause",
        "cavale",
        "cave",
        "ceci",
        "cela",
        "celui",
        "cendre",
        "cent",
        "cependant",
        "cercle",
        "cerise",
        "cerner",
        "certes",
        "cerveau",
        "cesser",
        "chacun",
        "chair",
        "chaleur",
        "chamois",
        "chanson",
        "chaque",
        "charge",
        "chasse",
        "chat",
        "chaud",
        "chef",
        "chemin",
        "cheveu",
        "chez",
        "chicane",
        "chien",
        "chiffre",
        "chiner",
        "chiot",
        "chlore",
        "choc",
        "choix",
        "chose",
        "chou",
        "chute",
        "cibler",
        "cidre",
        "ciel",
        "cigale",
        "cinq",
        "cintre",
        "cirage",
        "cirque",
        "ciseau",
        "citation",
        "citer",
        "citron",
        "civet",
        "clairon",
        "clan",
        "classe",
        "clavier",
        "clef",
        "climat",
        "cloche",
        "cloner",
        "clore",
        "clos",
        "clou",
        "club",
        "cobra",
        "cocon",
        "coiffer",
        "coin",
        "colline",
        "colon",
        "combat",
        "comme",
        "compte",
        "conclure",
        "conduire",
        "confier",
        "connu",
        "conseil",
        "contre",
        "convenir",
        "copier",
        "cordial",
        "cornet",
        "corps",
        "cosmos",
        "coton",
        "couche",
        "coude",
        "couler",
        "coupure",
        "cour",
        "couteau",
        "couvrir",
        "crabe",
        "crainte",
        "crampe",
        "cran",
        "creuser",
        "crever",
        "crier",
        "crime",
        "crin",
        "crise",
        "crochet",
        "croix",
        "cruel",
        "cuisine",
        "cuite",
        "culot",
        "culte",
        "cumul",
        "cure",
        "curieux",
        "cuve",
        "dame",
        "danger",
        "dans",
        "davantage",
        "debout",
        "dedans",
        "dehors",
        "delta",
        "demain",
        "demeurer",
        "demi",
        "dense",
        "dent",
        "depuis",
        "dernier",
        "descendre",
        "dessus",
        "destin",
        "dette",
        "deuil",
        "deux",
        "devant",
        "devenir",
        "devin",
        "devoir",
        "dicton",
        "dieu",
        "difficile",
        "digestion",
        "digue",
        "diluer",
        "dimanche",
        "dinde",
        "diode",
        "dire",
        "diriger",
        "discours",
        "disposer",
        "distance",
        "divan",
        "divers",
        "docile",
        "docteur",
        "dodu",
        "dogme",
        "doigt",
        "dominer",
        "donation",
        "donjon",
        "donner",
        "dopage",
        "dorer",
        "dormir",
        "doseur",
        "douane",
        "double",
        "douche",
        "douleur",
        "doute",
        "doux",
        "douzaine",
        "draguer",
        "drame",
        "drap",
        "dresser",
        "droit",
        "duel",
        "dune",
        "duper",
        "durant",
        "durcir",
        "durer",
        "eaux",
        "effacer",
        "effet",
        "effort",
        "effrayant",
        "elle",
        "embrasser",
        "emmener",
        "emparer",
        "empire",
        "employer",
        "emporter",
        "enclos",
        "encore",
        "endive",
        "endormir",
        "endroit",
        "enduit",
        "enfant",
        "enfermer",
        "enfin",
        "enfler",
        "enfoncer",
        "enfuir",
        "engager",
        "engin",
        "enjeu",
        "enlever",
        "ennemi",
        "ennui",
        "ensemble",
        "ensuite",
        "entamer",
        "entendre",
        "entier",
        "entourer",
        "entre",
        "envelopper",
        "envie",
        "envoyer",
        "erreur",
        "escalier",
        "espace",
        "espoir",
        "esprit",
        "essai",
        "essor",
        "essuyer",
        "estimer",
        "exact",
        "examiner",
        "excuse",
        "exemple",
        "exiger",
        "exil",
        "exister",
        "exode",
        "expliquer",
        "exposer",
        "exprimer",
        "extase",
        "fable",
        "facette",
        "facile",
        "fade",
        "faible",
        "faim",
        "faire",
        "fait",
        "falloir",
        "famille",
        "faner",
        "farce",
        "farine",
        "fatigue",
        "faucon",
        "faune",
        "faute",
        "faux",
        "faveur",
        "favori",
        "faxer",
        "feinter",
        "femme",
        "fendre",
        "fente",
        "ferme",
        "festin",
        "feuille",
        "feutre",
        "fiable",
        "fibre",
        "ficher",
        "fier",
        "figer",
        "figure",
        "filet",
        "fille",
        "filmer",
        "fils",
        "filtre",
        "final",
        "finesse",
        "finir",
        "fiole",
        "firme",
        "fixe",
        "flacon",
        "flair",
        "flamme",
        "flan",
        "flaque",
        "fleur",
        "flocon",
        "flore",
        "flot",
        "flou",
        "fluide",
        "fluor",
        "flux",
        "focus",
        "foin",
        "foire",
        "foison",
        "folie",
        "fonction",
        "fondre",
        "fonte",
        "force",
        "forer",
        "forger",
        "forme",
        "fort",
        "fosse",
        "fouet",
        "fouine",
        "foule",
        "four",
        "foyer",
        "frais",
        "franc",
        "frapper",
        "freiner",
        "frimer",
        "friser",
        "frite",
        "froid",
        "froncer",
        "fruit",
        "fugue",
        "fuir",
        "fuite",
        "fumer",
        "fureur",
        "furieux",
        "fuser",
        "fusil",
        "futile",
        "futur",
        "gagner",
        "gain",
        "gala",
        "galet",
        "galop",
        "gamme",
        "gant",
        "garage",
        "garde",
        "garer",
        "gauche",
        "gaufre",
        "gaule",
        "gaver",
        "gazon",
        "geler",
        "genou",
        "genre",
        "gens",
        "gercer",
        "germer",
        "geste",
        "gibier",
        "gicler",
        "gilet",
        "girafe",
        "givre",
        "glace",
        "glisser",
        "globe",
        "gloire",
        "gluant",
        "gober",
        "golf",
        "gommer",
        "gorge",
        "gosier",
        "goutte",
        "grain",
        "gramme",
        "grand",
        "gras",
        "grave",
        "gredin",
        "griffure",
        "griller",
        "gris",
        "gronder",
        "gros",
        "grotte",
        "groupe",
        "grue",
        "guerrier",
        "guetter",
        "guider",
        "guise",
        "habiter",
        "hache",
        "haie",
        "haine",
        "halte",
        "hamac",
        "hanche",
        "hangar",
        "hanter",
        "haras",
        "hareng",
        "harpe",
        "hasard",
        "hausse",
        "haut",
        "havre",
        "herbe",
        "heure",
        "hibou",
        "hier",
        "histoire",
        "hiver",
        "hochet",
        "homme",
        "honneur",
        "honte",
        "horde",
        "horizon",
        "hormone",
        "houle",
        "housse",
        "hublot",
        "huile",
        "huit",
        "humain",
        "humble",
        "humide",
        "humour",
        "hurler",
        "idole",
        "igloo",
        "ignorer",
        "illusion",
        "image",
        "immense",
        "immobile",
        "imposer",
        "impression",
        "incapable",
        "inconnu",
        "index",
        "indiquer",
        "infime",
        "injure",
        "inox",
        "inspirer",
        "instant",
        "intention",
        "intime",
        "inutile",
        "inventer",
        "inviter",
        "iode",
        "iris",
        "issue",
        "ivre",
        "jade",
        "jadis",
        "jamais",
        "jambe",
        "janvier",
        "jardin",
        "jauge",
        "jaunisse",
        "jeter",
        "jeton",
        "jeudi",
        "jeune",
        "joie",
        "joindre",
        "joli",
        "joueur",
        "journal",
        "judo",
        "juge",
        "juillet",
        "juin",
        "jument",
        "jungle",
        "jupe",
        "jupon",
        "jurer",
        "juron",
        "jury",
        "jusque",
        "juste",
        "kayak",
        "ketchup",
        "kilo",
        "kiwi",
        "koala",
        "label",
        "lacet",
        "lacune",
        "laine",
        "laisse",
        "lait",
        "lame",
        "lancer",
        "lande",
        "laque",
        "lard",
        "largeur",
        "larme",
        "larve",
        "lasso",
        "laver",
        "lendemain",
        "lentement",
        "lequel",
        "lettre",
        "leur",
        "lever",
        "levure",
        "liane",
        "libre",
        "lien",
        "lier",
        "lieutenant",
        "ligne",
        "ligoter",
        "liguer",
        "limace",
        "limer",
        "limite",
        "lingot",
        "lion",
        "lire",
        "lisser",
        "litre",
        "livre",
        "lobe",
        "local",
        "logis",
        "loin",
        "loisir",
        "long",
        "loque",
        "lors",
        "lotus",
        "louer",
        "loup",
        "lourd",
        "louve",
        "loyer",
        "lubie",
        "lucide",
        "lueur",
        "luge",
        "luire",
        "lundi",
        "lune",
        "lustre",
        "lutin",
        "lutte",
        "luxe",
        "machine",
        "madame",
        "magie",
        "magnifique",
        "magot",
        "maigre",
        "main",
        "mairie",
        "maison",
        "malade",
        "malheur",
        "malin",
        "manche",
        "manger",
        "manier",
        "manoir",
        "manquer",
        "marche",
        "mardi",
        "marge",
        "mariage",
        "marquer",
        "mars",
        "masque",
        "masse",
        "matin",
        "mauvais",
        "meilleur",
        "melon",
        "membre",
        "menacer",
        "mener",
        "mensonge",
        "mentir",
        "menu",
        "merci",
        "merlu",
        "mesure",
        "mettre",
        "meuble",
        "meunier",
        "meute",
        "miche",
        "micro",
        "midi",
        "miel",
        "miette",
        "mieux",
        "milieu",
        "mille",
        "mimer",
        "mince",
        "mineur",
        "ministre",
        "minute",
        "mirage",
        "miroir",
        "miser",
        "mite",
        "mixte",
        "mobile",
        "mode",
        "module",
        "moins",
        "mois",
        "moment",
        "momie",
        "monde",
        "monsieur",
        "monter",
        "moquer",
        "moral",
        "morceau",
        "mordre",
        "morose",
        "morse",
        "mortier",
        "morue",
        "motif",
        "motte",
        "moudre",
        "moule",
        "mourir",
        "mousse",
        "mouton",
        "mouvement",
        "moyen",
        "muer",
        "muette",
        "mugir",
        "muguet",
        "mulot",
        "multiple",
        "munir",
        "muret",
        "muse",
        "musique",
        "muter",
        "nacre",
        "nager",
        "nain",
        "naissance",
        "narine",
        "narrer",
        "naseau",
        "nasse",
        "nation",
        "nature",
        "naval",
        "navet",
        "naviguer",
        "navrer",
        "neige",
        "nerf",
        "nerveux",
        "neuf",
        "neutre",
        "neuve",
        "neveu",
        "niche",
        "nier",
        "niveau",
        "noble",
        "noce",
        "nocif",
        "noir",
        "nomade",
        "nombre",
        "nommer",
        "nord",
        "norme",
        "notaire",
        "notice",
        "notre",
        "nouer",
        "nougat",
        "nourrir",
        "nous",
        "nouveau",
        "novice",
        "noyade",
        "noyer",
        "nuage",
        "nuance",
        "nuire",
        "nuit",
        "nulle",
        "nuque",
        "oasis",
        "objet",
        "obliger",
        "obscur",
        "observer",
        "obtenir",
        "obus",
        "occasion",
        "occuper",
        "ocre",
        "octet",
        "odeur",
        "odorat",
        "offense",
        "officier",
        "offrir",
        "ogive",
        "oiseau",
        "olive",
        "ombre",
        "onctueux",
        "onduler",
        "ongle",
        "onze",
        "opter",
        "option",
        "orageux",
        "oral",
        "orange",
        "orbite",
        "ordinaire",
        "ordre",
        "oreille",
        "organe",
        "orgie",
        "orgueil",
        "orient",
        "origan",
        "orner",
        "orteil",
        "ortie",
        "oser",
        "osselet",
        "otage",
        "otarie",
        "ouate",
        "oublier",
        "ouest",
        "ours",
        "outil",
        "outre",
        "ouvert",
        "ouvrir",
        "ovale",
        "ozone",
        "pacte",
        "page",
        "paille",
        "pain",
        "paire",
        "paix",
        "palace",
        "palissade",
        "palmier",
        "palpiter",
        "panda",
        "panneau",
        "papa",
        "papier",
        "paquet",
        "parc",
        "pardi",
        "parfois",
        "parler",
        "parmi",
        "parole",
        "partir",
        "parvenir",
        "passer",
        "pastel",
        "patin",
        "patron",
        "paume",
        "pause",
        "pauvre",
        "paver",
        "pavot",
        "payer",
        "pays",
        "peau",
        "peigne",
        "peinture",
        "pelage",
        "pelote",
        "pencher",
        "pendre",
        "penser",
        "pente",
        "percer",
        "perdu",
        "perle",
        "permettre",
        "personne",
        "perte",
        "peser",
        "pesticide",
        "petit",
        "peuple",
        "peur",
        "phase",
        "photo",
        "phrase",
        "piano",
        "pied",
        "pierre",
        "pieu",
        "pile",
        "pilier",
        "pilote",
        "pilule",
        "piment",
        "pincer",
        "pinson",
        "pinte",
        "pion",
        "piquer",
        "pirate",
        "pire",
        "piste",
        "piton",
        "pitre",
        "pivot",
        "pizza",
        "placer",
        "plage",
        "plaire",
        "plan",
        "plaque",
        "plat",
        "plein",
        "pleurer",
        "pliage",
        "plier",
        "plonger",
        "plot",
        "pluie",
        "plume",
        "plus",
        "pneu",
        "poche",
        "podium",
        "poids",
        "poil",
        "point",
        "poire",
        "poison",
        "poitrine",
        "poivre",
        "police",
        "pollen",
        "pomme",
        "pompier",
        "poncer",
        "pondre",
        "pont",
        "portion",
        "poser",
        "position",
        "possible",
        "poste",
        "potage",
        "potin",
        "pouce",
        "poudre",
        "poulet",
        "poumon",
        "poupe",
        "pour",
        "pousser",
        "poutre",
        "pouvoir",
        "prairie",
        "premier",
        "prendre",
        "presque",
        "preuve",
        "prier",
        "primeur",
        "prince",
        "prison",
        "priver",
        "prix",
        "prochain",
        "produire",
        "profond",
        "proie",
        "projet",
        "promener",
        "prononcer",
        "propre",
        "prose",
        "prouver",
        "prune",
        "public",
        "puce",
        "pudeur",
        "puiser",
        "pull",
        "pulpe",
        "puma",
        "punir",
        "purge",
        "putois",
        "quand",
        "quartier",
        "quasi",
        "quatre",
        "quel",
        "question",
        "queue",
        "quiche",
        "quille",
        "quinze",
        "quitter",
        "quoi",
        "rabais",
        "raboter",
        "race",
        "racheter",
        "racine",
        "racler",
        "raconter",
        "radar",
        "radio",
        "rafale",
        "rage",
        "ragot",
        "raideur",
        "raie",
        "rail",
        "raison",
        "ramasser",
        "ramener",
        "rampe",
        "rance",
        "rang",
        "rapace",
        "rapide",
        "rapport",
        "rarement",
        "rasage",
        "raser",
        "rasoir",
        "rassurer",
        "rater",
        "ratio",
        "rature",
        "ravage",
        "ravir",
        "rayer",
        "rayon",
        "rebond",
        "recevoir",
        "recherche",
        "record",
        "reculer",
        "redevenir",
        "refuser",
        "regard",
        "regretter",
        "rein",
        "rejeter",
        "rejoindre",
        "relation",
        "relever",
        "religion",
        "remarquer",
        "remettre",
        "remise",
        "remonter",
        "remplir",
        "remuer",
        "rencontre",
        "rendre",
        "renier",
        "renoncer",
        "rentrer",
        "renverser",
        "repas",
        "repli",
        "reposer",
        "reproche",
        "requin",
        "respect",
        "ressembler",
        "reste",
        "retard",
        "retenir",
        "retirer",
        "retour",
        "retrouver",
        "revenir",
        "revoir",
        "revue",
        "rhume",
        "ricaner",
        "riche",
        "rideau",
        "ridicule",
        "rien",
        "rigide",
        "rincer",
        "rire",
        "risquer",
        "rituel",
        "rivage",
        "rive",
        "robe",
        "robot",
        "robuste",
        "rocade",
        "roche",
        "rodeur",
        "rogner",
        "roman",
        "rompre",
        "ronce",
        "rondeur",
        "ronger",
        "roque",
        "rose",
        "rosir",
        "rotation",
        "rotule",
        "roue",
        "rouge",
        "rouler",
        "route",
        "ruban",
        "rubis",
        "ruche",
        "rude",
        "ruelle",
        "ruer",
        "rugby",
        "rugir",
        "ruine",
        "rumeur",
        "rural",
        "ruse",
        "rustre",
        "sable",
        "sabot",
        "sabre",
        "sacre",
        "sage",
        "saint",
        "saisir",
        "salade",
        "salive",
        "salle",
        "salon",
        "salto",
        "salut",
        "salve",
        "samba",
        "sandale",
        "sanguin",
        "sapin",
        "sarcasme",
        "satisfaire",
        "sauce",
        "sauf",
        "sauge",
        "saule",
        "sauna",
        "sauter",
        "sauver",
        "savoir",
        "science",
        "scoop",
        "score",
        "second",
        "secret",
        "secte",
        "seigneur",
        "sein",
        "seize",
        "selle",
        "selon",
        "semaine",
        "sembler",
        "semer",
        "semis",
        "sensuel",
        "sentir",
        "sept",
        "serpe",
        "serrer",
        "sertir",
        "service",
        "seuil",
        "seulement",
        "short",
        "sien",
        "sigle",
        "signal",
        "silence",
        "silo",
        "simple",
        "singe",
        "sinon",
        "sinus",
        "sioux",
        "sirop",
        "site",
        "situation",
        "skier",
        "snob",
        "sobre",
        "social",
        "socle",
        "sodium",
        "soigner",
        "soir",
        "soixante",
        "soja",
        "solaire",
        "soldat",
        "soleil",
        "solide",
        "solo",
        "solvant",
        "sombre",
        "somme",
        "somnoler",
        "sondage",
        "songeur",
        "sonner",
        "sorte",
        "sosie",
        "sottise",
        "souci",
        "soudain",
        "souffrir",
        "souhaiter",
        "soulever",
        "soumettre",
        "soupe",
        "sourd",
        "soustraire",
        "soutenir",
        "souvent",
        "soyeux",
        "spectacle",
        "sport",
        "stade",
        "stagiaire",
        "stand",
        "star",
        "statue",
        "stock",
        "stop",
        "store",
        "style",
        "suave",
        "subir",
        "sucre",
        "suer",
        "suffire",
        "suie",
        "suite",
        "suivre",
        "sujet",
        "sulfite",
        "supposer",
        "surf",
        "surprendre",
        "surtout",
        "surveiller",
        "tabac",
        "table",
        "tabou",
        "tache",
        "tacler",
        "tacot",
        "tact",
        "taie",
        "taille",
        "taire",
        "talon",
        "talus",
        "tandis",
        "tango",
        "tanin",
        "tant",
        "taper",
        "tapis",
        "tard",
        "tarif",
        "tarot",
        "tarte",
        "tasse",
        "taureau",
        "taux",
        "taverne",
        "taxer",
        "taxi",
        "tellement",
        "temple",
        "tendre",
        "tenir",
        "tenter",
        "tenu",
        "terme",
        "ternir",
        "terre",
        "test",
        "texte",
        "thym",
        "tibia",
        "tiers",
        "tige",
        "tipi",
        "tique",
        "tirer",
        "tissu",
        "titre",
        "toast",
        "toge",
        "toile",
        "toiser",
        "toiture",
        "tomber",
        "tome",
        "tonne",
        "tonte",
        "toque",
        "torse",
        "tortue",
        "totem",
        "toucher",
        "toujours",
        "tour",
        "tousser",
        "tout",
        "toux",
        "trace",
        "train",
        "trame",
        "tranquille",
        "travail",
        "trembler",
        "trente",
        "tribu",
        "trier",
        "trio",
        "tripe",
        "triste",
        "troc",
        "trois",
        "tromper",
        "tronc",
        "trop",
        "trotter",
        "trouer",
        "truc",
        "truite",
        "tuba",
        "tuer",
        "tuile",
        "turbo",
        "tutu",
        "tuyau",
        "type",
        "union",
        "unique",
        "unir",
        "unisson",
        "untel",
        "urne",
        "usage",
        "user",
        "usiner",
        "usure",
        "utile",
        "vache",
        "vague",
        "vaincre",
        "valeur",
        "valoir",
        "valser",
        "valve",
        "vampire",
        "vaseux",
        "vaste",
        "veau",
        "veille",
        "veine",
        "velours",
        "velu",
        "vendre",
        "venir",
        "vent",
        "venue",
        "verbe",
        "verdict",
        "version",
        "vertige",
        "verve",
        "veste",
        "veto",
        "vexer",
        "vice",
        "victime",
        "vide",
        "vieil",
        "vieux",
        "vigie",
        "vigne",
        "ville",
        "vingt",
        "violent",
        "virer",
        "virus",
        "visage",
        "viser",
        "visite",
        "visuel",
        "vitamine",
        "vitrine",
        "vivant",
        "vivre",
        "vocal",
        "vodka",
        "vogue",
        "voici",
        "voile",
        "voir",
        "voisin",
        "voiture",
        "volaille",
        "volcan",
        "voler",
        "volt",
        "votant",
        "votre",
        "vouer",
        "vouloir",
        "vous",
        "voyage",
        "voyou",
        "vrac",
        "vrai",
        "yacht",
        "yeti",
        "yeux",
        "yoga",
        "zeste",
        "zinc",
        "zone",
        "zoom",
    ]



# -*- coding: utf-8 -*-
from __future__ import unicode_literals
import codecs
import os
import re
import sys
from distutils.core import setup

from setuptools import find_packages

here = os.path.abspath(os.path.dirname(__file__))


def find_version(*parts):
    """
    Figure out version number without importing the package.
    https://packaging.python.org/guides/single-sourcing-package-version/
    """
    with codecs.open(os.path.join(here, *parts), "r", errors="ignore") as fp:
        version_file = fp.read()
    version_match = re.search(r"^__version__ = ['\"](.*)['\"]", version_file, re.M)
    if version_match:
        return version_match.group(1)
    raise RuntimeError("Unable to find version string.")


version = find_version("monero", "__init__.py")

setup(
    name="monero",
    version=version,
    description="A comprehensive Python module for handling Monero cryptocurrency",
    url="https://github.com/monero-ecosystem/monero-python/",
    long_description=open("README.rst", "rb").read().decode("utf-8"),
    install_requires=open("requirements.txt", "r").read().splitlines(),
    tests_require=open(
        "test_requirements_py{:d}.txt".format(sys.version_info.major), "r"
    )
    .read()
    .splitlines(),
    packages=find_packages(".", exclude=["tests"]),
    include_package_data=True,
    author="Michał Sałaban",
    author_email="michal@salaban.info",
    license="BSD-3-Clause",
    classifiers=[
        "Development Status :: 4 - Beta",
        "Intended Audience :: Developers",
        "License :: OSI Approved :: BSD License",
        "Operating System :: OS Independent",
        "Programming Language :: Python",
        "Programming Language :: Python :: 2.7",
        "Programming Language :: Python :: 3.6",
        "Programming Language :: Python :: 3.7",
        "Programming Language :: Python :: 3.8",
        "Programming Language :: Python :: 3.8",
        "Programming Language :: Python :: 3.9",
        "Programming Language :: Python :: 3.10",
        "Topic :: Software Development :: Libraries :: Python Modules",
    ],
    keywords="monero cryptocurrency",
    test_suite="tests",
)



from __future__ import unicode_literals
import binascii
from datetime import datetime
import json
import logging
import operator
import requests

from ... import exceptions
from ...account import Account
from ...address import address, Address, SubAddress
from ...numbers import from_atomic, to_atomic, PaymentID
from ...seed import Seed
from ...transaction import Transaction, IncomingPayment, OutgoingPayment
from .exceptions import RPCError, Unauthorized, MethodNotFound

_log = logging.getLogger(__name__)


class JSONRPCWallet(object):
    """
    JSON RPC backend for Monero wallet (``monero-wallet-rpc``)

    :param protocol: `http` or `https`
    :param host: host name or IP
    :param port: port number
    :param path: path for JSON RPC requests (should not be changed)
    :param user: username to authenticate with over RPC
    :param password: password to authenticate with over RPC
    :param timeout: request timeout
    :param verify_ssl_certs: verify ssl certs for request
    :param proxy_url: a proxy to use
    """

    _master_address = None

    def __init__(
        self,
        protocol="http",
        host="127.0.0.1",
        port=18088,
        path="/json_rpc",
        user="",
        password="",
        timeout=30,
        verify_ssl_certs=True,
        proxy_url=None,
    ):
        self.url = "{protocol}://{host}:{port}/json_rpc".format(
            protocol=protocol, host=host, port=port
        )
        _log.debug("JSONRPC wallet backend URL: {url}".format(url=self.url))
        self.auth = requests.auth.HTTPDigestAuth(user, password)
        self.session = requests.Session()
        self.timeout = timeout
        self.verify_ssl_certs = verify_ssl_certs
        self.proxies = {protocol: proxy_url}
        _log.debug(
            "JSONRPC wallet backend auth: '{user}'/'{stars}'".format(
                user=user, stars=("*" * len(password)) if password else ""
            )
        )

    def height(self):
        return self.raw_request("getheight")["height"]

    def spend_key(self):
        return self.raw_request("query_key", {"key_type": "spend_key"})["key"]

    def view_key(self):
        return self.raw_request("query_key", {"key_type": "view_key"})["key"]

    def seed(self):
        return Seed(self.raw_request("query_key", {"key_type": "mnemonic"})["key"])

    def accounts(self):
        accounts = []
        _accounts = self.raw_request("get_accounts")
        idx = 0
        self._master_address = Address(
            _accounts["subaddress_accounts"][0]["base_address"]
        )
        for _acc in _accounts["subaddress_accounts"]:
            assert idx == _acc["account_index"]
            accounts.append(
                Account(self, _acc["account_index"], label=_acc.get("label"))
            )
            idx += 1
        return accounts

    def new_account(self, label=None):
        _account = self.raw_request("create_account", {"label": label})
        # NOTE: the following should re-read label by _account.get('label') but the RPC
        # doesn't return that detail here
        return Account(self, _account["account_index"], label=label), SubAddress(
            _account["address"]
        )

    def addresses(self, account=0, addr_indices=None):
        qdata = {"account_index": account}
        if addr_indices:
            qdata["address_index"] = addr_indices
        _addresses = self.raw_request("getaddress", qdata)
        addresses = [None] * (
            max(map(operator.itemgetter("address_index"), _addresses["addresses"])) + 1
        )
        for _addr in _addresses["addresses"]:
            addresses[_addr["address_index"]] = address(
                _addr["address"], label=_addr.get("label", None)
            )
        return addresses

    def new_address(self, account=0, label=None):
        _address = self.raw_request(
            "create_address", {"account_index": account, "label": label}
        )
        return SubAddress(_address["address"]), _address["address_index"]

    def balances(self, account=0):
        _balance = self.raw_request("getbalance", {"account_index": account})
        return (
            from_atomic(_balance["balance"]),
            from_atomic(_balance["unlocked_balance"]),
        )

    def transfers_in(self, account, pmtfilter):
        params = {"account_index": account, "pending": False}
        method = "get_transfers"
        if pmtfilter.tx_ids:
            method = "get_transfer_by_txid"
        if pmtfilter.unconfirmed:
            params["in"] = pmtfilter.confirmed
            params["out"] = False
            params["pool"] = True
        else:
            if pmtfilter.payment_ids:
                method = "get_bulk_payments"
                params["payment_ids"] = list(map(str, pmtfilter.payment_ids))
            else:
                params["in"] = pmtfilter.confirmed
                params["out"] = False
                params["pool"] = False
        if method == "get_transfers":
            if pmtfilter.min_height:
                # NOTE: the API uses (min, max] range which is confusing
                params["min_height"] = pmtfilter.min_height - 1
                params["filter_by_height"] = True
            if pmtfilter.max_height:
                params["max_height"] = pmtfilter.max_height
                params["filter_by_height"] = True
            _pmts = self.raw_request(method, params)
            pmts = _pmts.get("in", [])
        elif method == "get_transfer_by_txid":
            pmts = []
            for txid in pmtfilter.tx_ids:
                params["txid"] = txid
                try:
                    _pmts = self.raw_request(method, params, squelch_error_logging=True)
                except exceptions.TransactionNotFound:
                    continue
                pmts.extend(_pmts["transfers"])
                # Issue #71: incoming payments to self will have excess 'destinations' key. Remove.
                for pmt in pmts:
                    try:
                        del pmt["destinations"]
                    except KeyError:
                        pass
        else:
            # NOTE: the API uses (min, max] range which is confusing
            params["min_block_height"] = (pmtfilter.min_height or 1) - 1
            _pmts = self.raw_request(method, params)
            pmts = _pmts.get("payments", [])
        if pmtfilter.unconfirmed:
            pmts.extend(_pmts.get("pool", []))
        return list(pmtfilter.filter(map(self._inpayment, pmts)))

    def transfers_out(self, account, pmtfilter):
        if pmtfilter.tx_ids:
            pmts = []
            for txid in pmtfilter.tx_ids:
                try:
                    _pmts = self.raw_request(
                        "get_transfer_by_txid",
                        {"account_index": account, "txid": txid},
                        squelch_error_logging=True,
                    )
                except exceptions.TransactionNotFound:
                    continue
                pmts.extend(_pmts["transfers"])
        else:
            _pmts = self.raw_request(
                "get_transfers",
                {
                    "account_index": account,
                    "in": False,
                    "out": pmtfilter.confirmed,
                    "pool": False,
                    "pending": pmtfilter.unconfirmed,
                },
            )
            pmts = _pmts.get("out", [])
            if pmtfilter.unconfirmed:
                pmts.extend(_pmts.get("pending", []))
        return list(pmtfilter.filter(map(self._outpayment, pmts)))

    def _paymentdict(self, data):
        pid = data.get("payment_id", None)
        laddr = data.get("address", None)
        if laddr:
            laddr = address(laddr)
        result = {
            "payment_id": None if pid is None else PaymentID(pid),
            "amount": from_atomic(data["amount"]),
            "timestamp": datetime.fromtimestamp(data["timestamp"])
            if "timestamp" in data
            else None,
            "note": data.get("note", None),
            "transaction": self._tx(data),
            "local_address": laddr,
        }
        if "destinations" in data:
            result["destinations"] = [
                (address(x["address"]), from_atomic(x["amount"]))
                for x in data.get("destinations")
            ]
        return result

    def _inpayment(self, data):
        return IncomingPayment(**self._paymentdict(data))

    def _outpayment(self, data):
        return OutgoingPayment(**self._paymentdict(data))

    def _tx(self, data):
        return Transaction(
            **{
                "hash": data.get("txid", data.get("tx_hash")),
                "fee": from_atomic(data["fee"]) if "fee" in data else None,
                "key": data.get("key"),
                "height": data.get("height", data.get("block_height")) or None,
                "timestamp": datetime.fromtimestamp(data["timestamp"])
                if "timestamp" in data
                else None,
                "blob": binascii.unhexlify(data.get("blob", "")),
                "confirmations": data.get("confirmations", None),
            }
        )

    def export_outputs(self):
        return self.raw_request("export_outputs")["outputs_data_hex"]

    def import_outputs(self, outputs_hex):
        return self.raw_request("import_outputs", {"outputs_data_hex": outputs_hex})[
            "num_imported"
        ]

    def export_key_images(self):
        return self.raw_request("export_key_images")["signed_key_images"]

    def import_key_images(self, key_images):
        _data = self.raw_request("import_key_images", {"signed_key_images": key_images})
        return (
            _data["height"],
            from_atomic(_data["spent"]),
            from_atomic(_data["unspent"]),
        )

    def transfer(
        self,
        destinations,
        priority,
        payment_id=None,
        unlock_time=0,
        account=0,
        relay=True,
    ):
        data = {
            "account_index": account,
            "destinations": list(
                map(
                    lambda dst: {
                        "address": str(address(dst[0])),
                        "amount": to_atomic(dst[1]),
                    },
                    destinations,
                )
            ),
            "priority": priority,
            "unlock_time": 0,
            "get_tx_keys": True,
            "get_tx_hex": True,
            "new_algorithm": True,
            "do_not_relay": not relay,
        }
        if payment_id is not None:
            data["payment_id"] = str(PaymentID(payment_id))
        _transfers = self.raw_request("transfer_split", data)
        _pertx = [
            dict(_tx)
            for _tx in map(
                lambda vs: zip(
                    ("txid", "amount", "fee", "key", "blob", "payment_id"), vs
                ),
                zip(
                    *[
                        _transfers[k]
                        for k in (
                            "tx_hash_list",
                            "amount_list",
                            "fee_list",
                            "tx_key_list",
                            "tx_blob_list",
                        )
                    ]
                ),
            )
        ]
        for d in _pertx:
            d["payment_id"] = payment_id
        return [self._tx(data) for data in _pertx]

    def sweep_all(
        self,
        destination,
        priority,
        payment_id=None,
        subaddr_indices=None,
        unlock_time=0,
        account=0,
        relay=True,
    ):
        if not subaddr_indices:
            # retrieve indices of all subaddresses with positive unlocked balance
            bals = self.raw_request("get_balance", {"account_index": account})
            subaddr_indices = []
            for subaddr in bals["per_subaddress"]:
                if subaddr.get("unlocked_balance", 0):
                    subaddr_indices.append(subaddr["address_index"])
        data = {
            "account_index": account,
            "address": str(address(destination)),
            "subaddr_indices": list(subaddr_indices),
            "priority": priority,
            "unlock_time": 0,
            "get_tx_keys": True,
            "get_tx_hex": True,
            "do_not_relay": not relay,
        }
        if payment_id is not None:
            data["payment_id"] = str(PaymentID(payment_id))
        _transfers = self.raw_request("sweep_all", data)
        _pertx = [
            dict(_tx)
            for _tx in map(
                lambda vs: zip(
                    ("txid", "amount", "fee", "key", "blob", "payment_id"), vs
                ),
                zip(
                    *[
                        _transfers[k]
                        for k in (
                            "tx_hash_list",
                            "amount_list",
                            "fee_list",
                            "tx_key_list",
                            "tx_blob_list",
                        )
                    ]
                ),
            )
        ]
        for d in _pertx:
            d["payment_id"] = payment_id
        return list(
            zip(
                [self._tx(data) for data in _pertx],
                map(from_atomic, _transfers["amount_list"]),
            )
        )

    def raw_request(self, method, params=None, squelch_error_logging=False):
        hdr = {"Content-Type": "application/json"}
        data = {"jsonrpc": "2.0", "id": 0, "method": method, "params": params or {}}
        _log.debug(
            "Method: {method}\nParams:\n{params}".format(
                method=method, params=json.dumps(params, indent=2, sort_keys=True)
            )
        )
        rsp = self.session.post(
            self.url,
            headers=hdr,
            data=json.dumps(data),
            auth=self.auth,
            timeout=self.timeout,
            verify=self.verify_ssl_certs,
            proxies=self.proxies,
        )

        if rsp.status_code == 401:
            raise Unauthorized("401 Unauthorized. Invalid RPC user name or password.")
        elif rsp.status_code != 200:
            raise RPCError(
                "Invalid HTTP status {code} for method {method}.".format(
                    code=rsp.status_code, method=method
                )
            )
        result = rsp.json()
        _ppresult = json.dumps(result, indent=2, sort_keys=True)
        _log.debug("Result:\n{result}".format(result=_ppresult))

        if "error" in result:
            err = result["error"]
            if not squelch_error_logging:
                _log.error("JSON RPC error:\n{result}".format(result=_ppresult))
            if err["code"] in _err2exc:
                raise _err2exc[err["code"]](err["message"])
            else:
                raise RPCError(
                    "Method '{method}' failed with RPC Error of unknown code {code}, "
                    "message: {message}".format(
                        method=method, data=data, result=result, **err
                    )
                )
        return result["result"]


_err2exc = {
    -2: exceptions.WrongAddress,
    -4: exceptions.GenericTransferError,
    -5: exceptions.WrongPaymentId,
    -8: exceptions.TransactionNotFound,
    -9: exceptions.SignatureCheckFailed,
    -14: exceptions.AccountIndexOutOfBound,
    -15: exceptions.AddressIndexOutOfBound,
    -16: exceptions.TransactionNotPossible,
    -17: exceptions.NotEnoughMoney,
    -20: exceptions.AmountIsZero,
    -29: exceptions.WalletIsWatchOnly,
    -37: exceptions.NotEnoughUnlockedMoney,
    -38: exceptions.NoDaemonConnection,
    -43: exceptions.WalletIsNotDeterministic,  # https://github.com/monero-project/monero/pull/4653
    -32601: MethodNotFound,
}
